package net.sf.jolene.constants;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Map;

/**
 * Defined preferences for jolene.properties file. TODO maybe conver to Enum.
 *
 * @author Dan Howard
 * @since Dec 31, 2005 10:40 PM
 */
public class Prefs {

    private static final Logger log = LogManager.getLogger(Prefs.class);

    /**
     * Default: 32. The initial size of the internal maps used to store documents.
     *
     * @see net.sf.jolene.factories.DocumentFactory
     */
    public static final Prefs CacheSize = new Prefs("CacheSize", 32);

    /**
     * Default: true. Indicates whether or not document cacheing is enabled. Cached documents
     * are read once when requested and copies are returned from the DocumentFactory. If cacheing
     * is disabled then documents are read and parsed whenever they are requested.
     *
     * @see net.sf.jolene.factories.DocumentFactory
     */
    public static final Prefs DocumentCaching = new Prefs("DocumentCaching", true);

    /**
     * When set to true /resources/application.properites file is updated based on the template labels.
     */
    public static final Prefs GenerateLocaleStrings = new Prefs("GenerateLocaleStrings", false);

    /**
     * Default comma (,). Used by Multi-Select objects when setting or getting option values.
     */
    public static final Prefs SelectValueSeperator = new Prefs("SelectValueSeperator", ",");

    /**
     * Default true. Indicates whether Label objects should stream back their label tags. You
     * might want to set this to false if you intend to use Labels to stream back arbitrary
     * HTML or JavaScript.  In these cases you could also use the Text object.
     *
     * @see net.sf.jolene.dom.Text
     */
    public static final Prefs StreamLabelTags = new Prefs("StreamLabelTags", true);

    /**
     * Default false. Indicates whether or not the streaming engine and the string utilities should
     * output in XHTML or HTML format.  It does not affect the parser.
     */
    public static final Prefs XHTMLStrict = new Prefs("XHTMLStrict", false);

    private static Map<String, Prefs> prefMap = null;


    private final String name;
    private Object value;

    private Prefs(String name, Object value) {
        this.name = name;
        this.value = value;
    }

    @Override
    public String toString() {
        return name;
    }

    public String getValue() {
        return value.toString();
    }

    public void setValue(Object value) {
        this.value = value;
    }

    public static Prefs valueOf(String name) throws IllegalArgumentException {
        if (name == null) {
            throw new IllegalArgumentException("valueOf parameter is null");
        }

        if (prefMap == null) {
            initPrefMap();
        }
        return prefMap.get(name.toLowerCase());
    }

    /**
     * Create a map using the pref strings as keys.
     */
    private static void initPrefMap() {
        Object o = new Object();
        synchronized (o) {
            prefMap = new HashMap<String, Prefs>(12);

            Field[] fields = Prefs.class.getFields();
            for (int j = 0; j < fields.length; j++) {

                try {
                    if (Prefs.class.equals(fields[j].getType())) {
                        prefMap.put(fields[j].getName().toLowerCase(), (Prefs) fields[j].get(j));
                    }
                } catch (IllegalAccessException e) {
                    log.error(e.getMessage(), e);
                }
            }
        }
    }
}

