package net.sf.jolene.dom;

import net.sf.jolene.constants.Tags;

/**
 * A button in a html document using the button - not the input tag.
 *
 * @author Dan Howard
 * @since Nov 16, 2003
 */
public final class Button extends HTMLElement {

    /**
     * Construct a button with the specified name.
     *
     * @param name name of the button.
     */
    public Button(String name) {
        this();
        setName(name);
    }

    /**
     * Default constructor.
     */
    public Button() {
        tag = Tags.button;
    }

    /**
     * Returns a clone of the button object.
     *
     * @return Button object.
     */
    @Override
    public Button clone() {
        return (Button) super.clone();
    }

    /**
     * Sets the name of the button.
     *
     * @param name name of the button.
     */
    @Override
    public void setName(String name) {
        super.setName(name);
        setAttribute("name", name);
    }

    /**
     * Sets the value of the button.
     *
     * @param value value of the button.
     */
    @Override
    public void setValue(String value) {
        super.setValue(value);
        setAttribute("value", value);
    }

    /**
     * Renders the button.
     *
     * @return The button as a html string.
     */
    @Override
    public String toString() {
        if (swapWith != null) {
            return swapWith.toString();
        }

        if (!isRenderable() && !log.isDebugEnabled()) {
            return "";
        }

        return super.toString() + getContent() + "</" + tag + '>';
    }
}
