package net.sf.jolene.dom;

import net.sf.jolene.constants.Tags;

/**
 * @author Dan Howard
 * @since Oct 10, 2005 8:48:27 AM
 */
final class GridCell extends HTMLElement {

    private boolean header = false;
    private HTMLElement cellObject = null;

    GridCell() {
        tag = Tags.td;
    }

    GridCell(GridColumn column) {
        this();

        if (column.getWidth().length() > 0) {
            setAttribute("width", column.getWidth());
        }

        if (column.getAlign().length() > 0) {
            setAttribute("align", column.getAlign());
        }

        cellObject = column.getCellObject();

        setValue(column.getHeader());

    }

    @Override
    public String toString() {
        if (swapWith != null) {
            return swapWith.toString();
        }

        if (header) {
            tag = Tags.th;
        }
        String renderString;

        if (cellObject == null || Tags.th == tag) {
            renderString = super.toString() + getValue() + "</" + tag + '>';
        } else {
            renderString = super.toString() + cellObject + "</" + tag + '>';
        }

        return renderString;
    }

    boolean isHeader() {
        return header;
    }

    void setHeader(boolean header) {
        this.header = header;
    }


    @Override
    public void setValue(String value) {
        super.setValue(value);

        if (cellObject != null) {
            cellObject.setValue(value);
        }
    }


    void setCellObject(HTMLElement cellObject) {
        this.cellObject = cellObject;
    }


    HTMLElement getCellObject() {
        return cellObject;
    }


    @Override
    public GridCell clone() {
        return (GridCell) super.clone();
    }
}
