package net.sf.jolene.dom;

import net.sf.jolene.constants.Tags;

/**
 * A TextArea in a html document.
 *
 * @author Dan Howard
 * @since Oct 11, 2003
 */
public final class TextArea extends HTMLElement {

    /**
     * Default constructor.
     */
    public TextArea() {
        tag = Tags.textarea;
    }


    /**
     * Construct a textarea with the specified name.
     *
     * @param name name of the textarea.
     */
    public TextArea(String name) {
        this();
        setName(name);
    }


    /**
     * Returns a clone of the textarea object.
     *
     * @return TextArea object.
     */
    @Override
    public TextArea clone() {
        return (TextArea) super.clone();
    }


    /**
     * Sets the text of the textarea. Same as setValue.
     *
     * @param text text value to set on TextArea.
     */
    @Override
    public void setContent(String text) {
        super.setContent(text);
        setValue(text);
    }

    /**
     * Sets the name of the textarea.
     *
     * @param name name of the textarea.
     */
    @Override
    public void setName(String name) {
        super.setName(name);
        setAttribute("name", name);
    }

    /**
     * Renders the textarea.
     *
     * @return The textarea as a html string.
     */
    @Override
    public String toString() {

        if (swapWith != null) {
            return swapWith;
        }

        if (!isRenderable() && !log.isDebugEnabled()) {
            return "";
        }

        return super.toString() + getValue() + "</" + tag + '>';
    }
}
