/*
 * Created on Dec 26, 2003
*/
package net.sf.jolene.html;

import java.util.*;


/**
 * Map wrapper to handle html attributes. It's a case insensitive sorted map.
 *
 * @author Dan Howard
 */
public class Attributes implements IAttributes {

    private SortedMap<String, String> attributes;

    /**
     *
     */
    public Attributes() {
        attributes = new TreeMap<String, String>();
    }

    /**
     * Sets an attribute.
     *
     * @param key   - attribute name
     * @param value - attribute value
     * @return String
     */
    public String setAttribute(String key, String value) {
        // Don't allow null values
        if (value == null) {
            value = "";
        }
        return attributes.put(key.toLowerCase(), value);
    }

    /**
     * Gets an attribute from the attribute map or NULL if not found.
     *
     * @param key - attribute name
     * @return String or NULL of key is null or not found
     */
    public String getAttribute(String key) {
        if (key == null) {
            return null;
        }
        return attributes.get(key.toLowerCase());
    }

    /**
     * Remove an attribute.
     *
     * @param key - attribute name
     * @return String
     */
    public String removeAttribute(String key) {
        return attributes.remove(key.toLowerCase());
    }

    /**
     * Indicates whether or not an attribute exists.
     *
     * @param key - attribute name
     * @return boolean
     */
    public boolean hasAttribute(String key) {
        if (key == null) {
            return false;
        }
        return getAttribute(key.toLowerCase()) != null;
    }

    /**
     * Returns the attributes as a Set.
     *
     * @return Set
     */
    public Set<String> keySet() {
        return attributes.keySet();
    }

    /**
     * Returns the attribute values as a Collection.
     *
     * @return Collection
     */
    public Collection<String> values() {
        return attributes.values();
    }


    /**
     * Clears the attributes.
     */
    public void clear() {
        attributes.clear();
    }

    @Override
    public String toString() {
        Iterator<String> it = keySet().iterator();
        StringBuilder s = new StringBuilder("");
        while (it.hasNext()) {
            String key = it.next();
            s.append(key).append('=').append(getAttribute(key)).append(' ');
        }
        return s.toString();
    }
}
