package net.sf.jolene.servlet;

import net.sf.jolene.util.PrefsReader;
import net.sf.jolene.dom.Document;
import net.sf.jolene.factories.DocumentFactory;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author Dan Howard
 * @since Aug 7, 2005 8:21:03 AM
 */
public class Domlet extends HttpServlet {

    private static final Logger log = LogManager.getLogger(Domlet.class);

    /**
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        handleRequest(request, response);
    }

    /**
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @throws ServletException
     * @throws IOException
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        handleRequest(request, response);
    }

    /**
     * @throws ServletException
     */
    @Override
    public void init() throws ServletException {
        // Initialize PrefsReader
        log.info("Domlet Initializing PrefsReader...");
        PrefsReader.init();
    }

    /**
     * Basic request handler for servlet requests. Looks at the the request or session to locate the
     * document object to stream out the response writer.
     *
     * @param request  HttpServletRequest
     * @param response HttpServletResponse
     * @throws IOException if an IOException occurs
     */
    protected static void handleRequest(HttpServletRequest request, HttpServletResponse response) throws IOException {

        log.debug("handle request");
        log.debug("URI: " + request.getRequestURI());
        log.debug("URL: " + request.getRequestURL());
        log.debug("CP : " + request.getContextPath());
        log.debug("SP : " + request.getServletPath());

        // look 1st for the document in the request scope
        // otherwise look in the session scope
        // if none found log warn and add to request scope
        // The key is the URI minus the context path
        String uri = request.getRequestURI();
        if (request.getContextPath().length() > 0) {
            if (uri.startsWith(request.getContextPath())) {
                int n;
                n = uri.indexOf("/", 1);
                if (n > -1) {
                    uri = uri.substring(n);
                }
            }
        }

        Document document;
        if (request.getAttribute(uri) != null) {
            document = (Document) request.getAttribute(uri);
        } else {
            document = (Document) request.getSession().getAttribute(uri);
        }

        if (document == null) {
            log.warn("No document found at request or session level for url " + uri + " attaching a new document to the request.");
            document = DocumentFactory.getInstance().getDocument(request.getSession().getServletContext().getRealPath(uri), request.getContextPath(), uri);
        }
        log.debug("DOC URI: " + document.getUri());

        document.stream(response.getWriter());
    }


}
