package domlet.kbsample.managers;

import domlet.kbsample.data.Article;
import domlet.kbsample.util.StringUtil;
import domlet.kbsample.web.viewValues.ArticleView;
import domlet.kbsample.managers.constants.VisibilityTypes;
import net.sf.persist.Persist;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Oct 30, 2005
 * Time: 9:04:39 AM
 */
public class ArticleManager {

    private static final Logger log = LogManager.getLogger(ArticleManager.class);

    private IContextInfo contextInfo;
    private Persist persist;

    ArticleManager(IContextInfo contextInfo) {
        this.contextInfo = contextInfo;
        persist = new Persist(contextInfo.getConnection());
    }


    public List<ArticleView> getArticles(ArticleQueryInfo queryInfo) {

        List<Object> params = new ArrayList<Object>();
        StringBuilder where = new StringBuilder();

        if (queryInfo.getCategory() > 0) {
            params.add(queryInfo.getCategory());
            where.append("category_id=?");
        }

        if (StringUtil.isNotEmpty(queryInfo.getCreator())) {
            params.add('%' + queryInfo.getCreator() + '%');
            if (where.length() > 0) {
                where.append(" AND ");
            }
            where.append("c.name like ?");
        }

        if (StringUtil.isNotEmpty(queryInfo.getModifier())) {
            params.add('%' + queryInfo.getModifier() + '%');
            if (where.length() > 0) {
                where.append(" AND ");
            }
            where.append("m.name like ?");
        }

        if (StringUtil.isNotEmpty(queryInfo.getSubject())) {
            params.add('%' + queryInfo.getSubject() + '%');
            if (where.length() > 0) {
                where.append(" AND ");
            }
            where.append("subject like ?");
        }


        Calendar endTime = Calendar.getInstance();
        if (queryInfo.getModifiedEnd() != null) {
            endTime.setTime(queryInfo.getModifiedEnd());
            endTime.set(Calendar.HOUR, 23);
            endTime.set(Calendar.MINUTE, 59);
        }

        if (queryInfo.getModifiedStart() != null && queryInfo.getModifiedEnd() != null) {

            params.add(queryInfo.getModifiedStart());
            params.add(endTime.getTime());

            if (where.length() > 0) {
                where.append(" AND ");
            }
            where.append("(modified between ? and ?)");

        } else if (queryInfo.getModifiedStart() != null) {
            // no end specified
            params.add(queryInfo.getModifiedStart());
            if (where.length() > 0) {
                where.append(" AND ");
            }
            where.append("modified >= ?");


        } else if (queryInfo.getModifiedEnd() != null) {

            // no start specified
            params.add(endTime.getTime());
            if (where.length() > 0) {
                where.append(" AND ");
            }
            where.append("modified <= ?");

        }


        String sql = getArticleQuery();

        log.info(sql + where + " - " + params);

        if (where.length() > 0) {
            sql += " WHERE " + where;
        }
        sql += " ORDER BY subject";

        return persist.readList(ArticleView.class, sql, params.toArray());

    }

    private String getArticleQuery() {
        String sql = "select a.id, category_id, subject, article, created, creator_id, modified, modifier_id, security, users, active, c.name creatorName, m.name modifierName " +
                "from KB_articles a " +
                "left join kb_users c on creator_id=c.id " +
                "left join kb_users m on modifier_id=m.id ";
        return sql;
    }


    public Article getArticle(int id) {
        return persist.read(Article.class, "select * from KB_ARTICLES where id=?", id);
    }

    public ArticleView getArticleView(int id) {
        return persist.read(ArticleView.class, getArticleQuery() + " where a.id = ?", id);
    }

    public void saveArticle(Article article) {
        // If the user has not chosen specific users then make sure to blank the users field.
        if (!VisibilityTypes.Specify.toString().equals(article.getSecurity())) {
            article.setUsers("");
        }
        if (article.getId() <= 0) {
            persist.insert(article);
        } else {
            persist.update(article);
        }
    }

    public void deleteArticle(int id) {
        Article article = getArticle(id);
        if (article != null) {
            persist.delete(article);
        } else {
            log.warn("deleteArticle Article " + id + " not found.");
        }
    }

    public void deleteArticles(int[] articleIds) {
        // Lame/slow way.
        for (int j = 0; j < articleIds.length; j++) {
            deleteArticle(articleIds[j]);
        }
    }
}
