package domlet.kbsample.managers;

import domlet.kbsample.data.Category;
import domlet.kbsample.web.viewValues.CategoryView;
import net.sf.persist.Persist;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Oct 30, 2005
 * Time: 9:04:57 AM
 */
public class CategoryManager {

    private static final Logger log = LogManager.getLogger(CategoryManager.class);

    IContextInfo contextInfo;
    Persist persist;

    CategoryManager(IContextInfo contextInfo) {
        this.contextInfo = contextInfo;
        persist = new Persist(contextInfo.getConnection());
    }


    public List<Category> getCategories() {
        return persist.readList(Category.class, "select * from KB_CATEGORIES order by category");
    }

    public List<CategoryView> getCategoryViews() {
        List<Category> list = getCategories();
        Iterator<Category> it = list.iterator();

        List<CategoryView> categories = new ArrayList<CategoryView>(list.size());
        CategoryView value;
        while (it.hasNext()) {
            value = new CategoryView();
            try {
                BeanUtils.copyProperties(value, it.next());
                categories.add(value);
            } catch (IllegalAccessException e) {
                log.error(e.getMessage(), e);
            } catch (InvocationTargetException e) {
                log.error(e.getMessage(), e);
            }
        }
        return categories;
    }

    public boolean isSafeToDelete(int categoryId) {

        Map<String, Object> map = persist.readMap("select count(*) as count from KB_ARTICLES where category_id=?", categoryId);
        return 0L == (Long) map.get("count");
    }

    public void deleteCategory(int categoryId) {
        persist.delete(getCategory(categoryId));
    }

    public Category getCategory(int id) {
        return persist.read(Category.class, "select * from KB_CATEGORIES where id=?", id);
    }

    public void saveCategory(Category category) {

        if (category.getId() <= 0) {
            persist.insert(category);
        } else {
            persist.update(category);
        }
    }

    // Slow hiberbate way....
    public void saveCategories(Map<String, String> categories) {
        for (String id : categories.keySet()) {
            Category category = getCategory(Integer.parseInt(id));
            category.setCategory(categories.get(id));
            saveCategory(category);
        }
    }
}
