package domlet.kbsample.managers;

import domlet.kbsample.data.User;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.h2.jdbcx.JdbcConnectionPool;
import org.h2.jdbcx.JdbcDataSource;

import javax.servlet.http.HttpServletRequest;
import java.sql.Connection;
import java.sql.SQLException;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Dec 29, 2005
 * Time: 6:27:15 AM
 */
public class ContextInfo implements IContextInfo {

    private static final Logger log = LogManager.getLogger(ContextInfo.class);

    private User user;
    private static JdbcDataSource ds;
    private static JdbcConnectionPool cp;
    private Connection con;

    private ContextInfo() {
        synchronized (ContextInfo.class) {
            if (ds == null) {
                ds = new JdbcDataSource();
                ds.setURL("jdbc:h2:~/kbdata");
                ds.setUser("sa");
                ds.setPassword("");
                cp = JdbcConnectionPool.create(ds);
            }
            try {
                con = cp.getConnection();
            } catch (SQLException e) {
                log.error(e.getMessage(), e);
            }
        }
    }

    public static ContextInfo getInstance(HttpServletRequest request) {

        if (request.getAttribute("contextInfo") == null) {

            ContextInfo info = new ContextInfo();
            info.user = (User) request.getSession().getAttribute("user");

            request.setAttribute("contextInfo", info);
        }


        return (ContextInfo) request.getAttribute("contextInfo");
    }

    public void cleanUp() {
        try {
            if (con != null) {
                con.close();
            }
        } catch (SQLException e) {
            log.error(e.getMessage(), e);
        }
    }


    public Connection getConnection() {
        return con;
    }

    public User getUser() {
        return user;
    }


    public void setUser(User user) {
        this.user = user;
    }
}
