package domlet.kbsample.managers;

import domlet.kbsample.data.User;
import domlet.kbsample.web.viewValues.UserView;
import net.sf.persist.Persist;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Oct 30, 2005
 * Time: 9:05:23 AM
 */
public class UserManager {

    private static final Logger log = LogManager.getLogger(UserManager.class);

    IContextInfo contextInfo;
    Persist persist;

    UserManager(IContextInfo contextInfo) {
        this.contextInfo = contextInfo;
        persist = new Persist(contextInfo.getConnection());
    }

    public User getUser(int userId) {
        return persist.read(User.class, "select * from KB_USERS where id=?", userId);
    }


    public List<User> getUsers() {
        return persist.readList(User.class, "select * from KB_USERS order by name");
    }

    public List<User> getUsersByUsername(String username) {
        return persist.readList(User.class, "select * from KB_USERS where username=?", username);
    }

    public User authenticateUser(String username, String password) {
        User user = persist.read(User.class, "select * from KB_USERS where username = ?", username);
        if (user != null) {
            if (user.getPassword().equals(password)) {
                return user;
            }
        }
        return null;
    }


    public List<UserView> getUserViews() {
        List<User> list = getUsers();
        Iterator<User> it = list.iterator();

        List<UserView> users = new ArrayList<UserView>(list.size());
        UserView value;
        while (it.hasNext()) {
            value = new UserView();
            try {
                BeanUtils.copyProperties(value, it.next());
                users.add(value);
            } catch (IllegalAccessException e) {
                log.error(e.getMessage(), e);
            } catch (InvocationTargetException e) {
                log.error(e.getMessage(), e);
            }
        }
        return users;
    }

    public void saveUser(User user) {
//        //@todo check for duped usernames
        if (user.getId() <= 0) {
            persist.insert(user);
        } else {
            persist.update(user);
        }
    }

    public boolean isSafeToDelete(int userId) {

        // todo test isSafeToDelete
        Map<String, Object> map = persist.readMap("select count(*) as count from KB_ARTICLES where creator_id=? or modifier_id=?", userId, userId);

        return 0 == (Integer) map.get("count");

    }

    public void deleteUser(int userId) {
        persist.delete(getUser(userId));
    }
}
