package domlet.kbsample.web.actions;

import domlet.kbsample.data.Article;
import domlet.kbsample.data.Category;
import domlet.kbsample.data.User;
import domlet.kbsample.managers.ArticleQueryInfo;
import domlet.kbsample.managers.ContextInfo;
import domlet.kbsample.managers.IContextInfo;
import domlet.kbsample.managers.Manager;
import domlet.kbsample.managers.constants.Rules;
import domlet.kbsample.web.forms.ArticleForm;
import domlet.kbsample.web.forms.ArticleSearchForm;
import domlet.kbsample.web.viewValues.ArticleView;
import net.sf.jolene.constants.Entities;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.dom.*;
import net.sf.jolene.struts.DomletAction;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.jmesa.facade.TableFacade;
import org.jmesa.facade.TableFacadeFactory;
import org.jmesa.view.html.component.HtmlTable;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Dec 29, 2005
 * Time: 11:31:51 AM
 */
public class ArticleAction extends DomletAction {

    private static final Logger log = LogManager.getLogger(ArticleAction.class);


    public ActionForward editArticle(ActionMapping mapping,
                                     ActionForm aform,
                                     HttpServletRequest request,
                                     HttpServletResponse response) throws IOException {

        ContextInfo contextInfo = ContextInfo.getInstance(request);
        try {
            Document document;
            ArticleForm form = (ArticleForm) aform;
            ArticleView article;
            document = getDocument(mapping, request);

            Header header = new Header(Tags.meta);
            header.setAfterText("<!-- hello -->"); 
            header.setAttribute("name", "description");
            header.setAttribute("content", "Edit an Article!");
            document.addHeader(header);

            if (form.getId() > 0) {
                article = Manager.getArticleManager(contextInfo).getArticleView(form.getId());
                if (article != null) {
                    document.forms(0).populate(article);
                    document.forms(0).elements("save").setContent("Save");
                    document.forms(0).elements("createdLabel").setContent("Created");
                } else {
                    document = getDocument(mapping, "error", request);
                    document.forms(0).elements("Message").setValue("Article not found.");
                    return mapping.findForward("error");
                }
            } else {
                // Note we can either populate a temporary Article Object OR we
                // can simply directly assign to the document form object.

                /* Method 1
                article = new ArticleView();
                article.setActive(true);
                article.setId(-1);
                article.setSecurity("Public");
                article.setCreator(contextInfo.getUser());
                article.setModifier(contextInfo.getUser());
                article.setCreatorId(contextInfo.getUser().getId());
                article.setModifierId(contextInfo.getUser().getId());
                document.forms(0).populate(article);
                */

                // Method 2
                // Map<String, HTMLElement>
                document.forms(0).getRadioGroup("security").get("Public").setChecked(true);
                document.forms(0).elements("id").setValue("-1");
                document.forms(0).elements("title").setValue("New Article");
            }

            initCategoryList(document, contextInfo);
            initUserList(document, contextInfo);

            if (!Rules.isAllowEditing(contextInfo)) {
                document.forms(0).elements("title").setValue("View Article");
                document.forms(0).elements("save").setRenderable(false);
            }

            return mapping.findForward("success");
        } finally {
            contextInfo.cleanUp();
        }

    }


    public ActionForward saveArticle(ActionMapping mapping,
                                     ActionForm aform,
                                     HttpServletRequest request,
                                     HttpServletResponse response) {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            ArticleForm form = (ArticleForm) aform;
            Article article;
            if (form.getId() <= 0) {
                article = new Article();
            } else {
                article = Manager.getArticleManager(contextInfo).getArticle(form.getId());
            }

            article.setArticle(form.getArticle());
            article.setCategoryId(form.getCategory());

            String[] ausers = form.getUsers();
            StringBuffer sb = new StringBuffer("");
            String sep = "";
            for (int j = 0; j < ausers.length; j++) {

                sb.append(sep).append(ausers[j]);
                sep = ",";
            }
            log.debug("LIST: " + sb);
            article.setUsers(sb.toString());

            if (article.getCreated() == null) {
                article.setCreated(new Date());
            }
            article.setCreatorId(contextInfo.getUser().getId());
            article.setId(form.getId());
            article.setModified(new Date());
            article.setModifierId(contextInfo.getUser().getId());
            article.setSecurity(form.getSecurity());
            article.setSubject(form.getSubject());
            article.setActive(form.isActive());
            Manager.getArticleManager(contextInfo).saveArticle(article);

        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");
    }

    public ActionForward deleteArticle(ActionMapping mapping,
                                       ActionForm actionForm,
                                       HttpServletRequest request,
                                       HttpServletResponse response) throws IOException {

        ContextInfo contextInfo = ContextInfo.getInstance(request);
        try {
            Document document;

            ArticleForm form = (ArticleForm) actionForm;
            if (contextInfo.getUser() == null) {
                document = getDocument(mapping, "error", request);
                document.forms(0).elements("message").setValue("You are not permiited to delete.");
                return mapping.findForward("error");
            }


            if (form.getId() > 0) {
                Manager.getArticleManager(contextInfo).deleteArticle(form.getId());
            } else {
                document = getDocument(mapping, "error", request);
                document.forms(0).elements("message").setValue("Article id not passed or negative.");
                return mapping.findForward("error");
            }


        } finally {
            contextInfo.cleanUp();
        }
        return mapping.findForward("success");
    }


    public ActionForward searchArticles(ActionMapping mapping,
                                        ActionForm aform,
                                        HttpServletRequest request,
                                        HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            ArticleQueryInfo queryInfo;
            ArticleSearchForm form = (ArticleSearchForm) aform;
            if (request.getSession().getAttribute("queryInfo") == null) {
                queryInfo = new ArticleQueryInfo();
                request.getSession().setAttribute("queryInfo", queryInfo);
            } else {
                queryInfo = (ArticleQueryInfo) request.getSession().getAttribute("queryInfo");
            }
            BeanUtils.copyProperties(queryInfo, form);

            Document document = getDocument(mapping, request);
            initCategoryList(document, contextInfo);

            // Populate the document fields from the form bean.
            document.forms(0).populate(form);

            return browseArticles(mapping, aform, request, response);
        } finally {
            contextInfo.cleanUp();
        }
    }


    public ActionForward deleteArticles(ActionMapping mapping,
                                        ActionForm aform,
                                        HttpServletRequest request,
                                        HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            ArticleSearchForm form = (ArticleSearchForm) aform;
            Manager.getArticleManager(contextInfo).deleteArticles(form.getSelectedItems());
            return searchArticles(mapping, aform, request, response);
        } finally {
            contextInfo.cleanUp();
        }
    }

    public ActionForward browseArticles(ActionMapping mapping,
                                ActionForm aform,
                                HttpServletRequest request,
                                HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            Document document = getDocument(mapping, request);
            initCategoryList(document, contextInfo);

            ArticleQueryInfo queryInfo;
            List articles;

            if (request.getSession().getAttribute("queryInfo") == null) {
                queryInfo = new ArticleQueryInfo();
                request.getSession().setAttribute("queryInfo", queryInfo);
                articles = new ArrayList(0);
            } else {
                queryInfo = (ArticleQueryInfo) request.getSession().getAttribute("queryInfo");
                articles = Manager.getArticleManager(contextInfo).getArticles(queryInfo);
            }


            initDefaultGrid(document, articles, contextInfo);

            // initJmesaGrid(document, articles, request, contextInfo);

            boolean allowEditing = Rules.isAllowEditing(contextInfo);

            if (!allowEditing) {
                document.forms(0).elements("deleteSelected").setRenderable(false);
                document.forms(0).elements("new").setRenderable(false);
            }

            return mapping.findForward("success");
        } finally {
            contextInfo.cleanUp();
        }
    }


    private void initCategoryList(Document document, ContextInfo contextInfo) {
        List categories = Manager.getCategoryManager(contextInfo).getCategories();
        Select select = (Select) document.forms(0).elements("category");
        select.clearOptions();
        Iterator it;
        it = categories.iterator();
        while (it.hasNext()) {
            Category category;
            category = (Category) it.next();
            select.addOption(new Option(category.getId(), category.getCategory()));
        }

    }

    private void initUserList(Document document, ContextInfo contextInfo) {
        List<User> users = Manager.getUserManager(contextInfo).getUsers();
        Select select = document.forms(0).getSelect("users");
        select.clearOptions();
        Iterator<User> it;
        it = users.iterator();
        while (it.hasNext()) {
            User user = it.next();
            select.addOption(new Option(user.getId(), user.getName()));
        }

    }

    private static void initJmesaGrid(Document document, List articles, HttpServletRequest request, IContextInfo contextInfo) {
        TableFacade tableFacade = TableFacadeFactory.createTableFacade("grid1", request);

        boolean allowEditing = Rules.isAllowEditing(contextInfo);

        if (allowEditing) {
            tableFacade.setColumnProperties("subject", "creator.name", "createdString", "modifiedString", "edit", "delete");
        } else {
            tableFacade.setColumnProperties("subject", "creator.name", "createdString", "modifiedString");
        }

        tableFacade.setItems(articles);

        HtmlTable table = (HtmlTable) tableFacade.getTable();
        table.getRow().getColumn("creator.name").setTitle("Author");
        table.getRow().getColumn("createdString").setTitle("Created");
        table.getRow().getColumn("modifiedString").setTitle("Modified");
        table.getTableRenderer().setWidth("100%");

        if (allowEditing) {
            table.getRow().getColumn("edit").setFilterable(false);
            table.getRow().getColumn("edit").setSortable(false);
            table.getRow().getColumn("delete").setFilterable(false);
            table.getRow().getColumn("delete").setSortable(false);
        }

        Grid grid = document.forms(0).getGrid("grid1");
        grid.swapWith(tableFacade.render());
    }

    private static void initDefaultGrid(Document document, List articles, IContextInfo contextInfo) throws Exception {
        Grid grid = (Grid) document.forms(0).elements("grid1");

//            grid.setEvenRowClass("even");
//            grid.setOddRowClass("odd");
        if (articles.size() == 0) {
            grid.setRenderable(false);
            return;
        }

        boolean allowEditing = Rules.isAllowEditing(contextInfo);

        GridColumn col;
        List<GridColumn> header;
        List<String> fields;

        header = new ArrayList<GridColumn>(7);
        if (allowEditing) {
            col = new GridColumn("Select", "5%");
            col.setCellObject(new CheckBox("selectedItems"));
            col.getCellObject().setAttribute("onclick", "doSelectedItem()");
            header.add(col);

            col = new GridColumn("Subject", "45%");
            header.add(col);

        } else {
            col = new GridColumn("Subject", "50%");
            header.add(col);
        }

        col = new GridColumn("Author", "20%");
        header.add(col);

        col = new GridColumn("Created", "10%");
        header.add(col);

        col = new GridColumn("Modified", "10%");
        header.add(col);

        fields = new ArrayList<String>(7);
        if (allowEditing) {
            fields.add("id");
        }
        fields.add("subject");
        fields.add("creatorName");
        fields.add("createdString");
        fields.add("modifiedString");

        if (allowEditing) {
            col = new GridColumn(Entities.space, "5%");
            col.setAlign("center");
            header.add(col);

            col = new GridColumn(Entities.space, "5%");
            col.setAlign("center");
            header.add(col);

            fields.add("edit");
            fields.add("delete");

        } else {
            col = new GridColumn(Entities.space, "10%");
            col.setAlign("center");
            header.add(col);

            fields.add("view");

            document.forms(0).elements("new").setRenderable(false);
        }

        grid.bind(header, fields, articles);
    }

    private static void initCategories(Document document, IContextInfo contextInfo) {
        List<Category> categories = Manager.getCategoryManager(contextInfo).getCategories();
        Select select = document.forms(0).getSelect("category");
        select.clearOptions();
        select.addOption(new Option("", "Select..."));
        for (Category category : categories) {
            select.addOption(new Option(category.getId(), category.getCategory()));
        }
    }


}
