package domlet.kbsample.web.actions;

import domlet.kbsample.data.Category;
import domlet.kbsample.managers.ContextInfo;
import domlet.kbsample.managers.Manager;
import domlet.kbsample.managers.constants.Rules;
import domlet.kbsample.web.forms.CategoryForm;
import domlet.kbsample.web.viewValues.CategoryView;
import net.sf.jolene.constants.Entities;
import net.sf.jolene.dom.Document;
import net.sf.jolene.dom.Grid;
import net.sf.jolene.dom.GridColumn;
import net.sf.jolene.dom.Input;
import net.sf.jolene.struts.DomletAction;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;


/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Feb 11, 2006
 * Time: 7:33:36 AM
 */
public class CategoryAction extends DomletAction {

    private static final Logger log = LogManager.getLogger(CategoryAction.class);

    public ActionForward browseCategories(ActionMapping mapping,
                                          ActionForm actionForm,
                                          HttpServletRequest request,
                                          HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            Document document = getDocument(mapping, request);
            if (!Rules.isAllowEditing(contextInfo)) {
                document.forms(0).elements("saveEdits").setRenderable(false);
            }

            List<CategoryView> list = Manager.getCategoryManager(contextInfo).getCategoryViews();
            CategoryForm form = (CategoryForm) actionForm;
            form.setList(list);

            Grid grid = (Grid) document.forms(0).elements("grid1");

            GridColumn col;
            List<GridColumn> header;
            List<String> fields;

            header = new ArrayList<GridColumn>(1);
            col = new GridColumn("Category", "90%");
            col.setAlign("left");
            header.add(col);

            fields = new ArrayList<String>(3);
            fields.add("category");
            if (Rules.isAllowEditing(contextInfo)) {
                col.setCellObject(new Input("indexedCategory(${id})"));
                col.getCellObject().setAttribute("style", "width:80%");

                col = new GridColumn(Entities.space, "5%");
                col.setAlign("center");
                header.add(col);
                fields.add("delete");

            } else {
                col.setWidth("100%");
                document.forms(0).elements("new").setRenderable(false);
            }

            grid.bind(header, fields, list);


        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");
    }

    public ActionForward editCategory(ActionMapping mapping,
                                      ActionForm aform,
                                      HttpServletRequest request,
                                      HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            CategoryForm form = (CategoryForm) aform;
            Document document = getDocument(mapping, request);
            if (form.getId() > 0) {

                Category category = Manager.getCategoryManager(contextInfo).getCategory(form.getId());
                if (category != null) {
                    document.forms(0).populate(category);
                } else {
                    return mapping.findForward("error");
                }

            } else {
                document.forms(0).elements("id").setValue("-1");
                document.forms(0).elements("title").setValue("New Category");
            }
        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");
    }

    public ActionForward saveCategory(ActionMapping mapping,
                                      ActionForm aform,
                                      HttpServletRequest request,
                                      HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            CategoryForm form = (CategoryForm) aform;
            Category category;
            if (form.getId() < 0) {
                category = new Category();
            } else {
                category = Manager.getCategoryManager(contextInfo).getCategory(form.getId());
            }
            if (category != null) {

                category.setId(form.getId());
                category.setCategory(form.getCategory());

            } else {
                //todo send to error page with a message.
                return mapping.findForward("error");

            }
            Manager.getCategoryManager(contextInfo).saveCategory(category);
        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");
    }

    public ActionForward deleteCategory(ActionMapping mapping,
                                        ActionForm aform,
                                        HttpServletRequest request,
                                        HttpServletResponse response) throws IOException {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {

            Document document = getDocument(mapping, request);
            CategoryForm form = (CategoryForm) aform;
            if (Manager.getCategoryManager(contextInfo).isSafeToDelete(form.getId())) {
                Manager.getCategoryManager(contextInfo).deleteCategory(form.getId());
                document.forms(0).elements("message").setValue("");
            } else {
                document.forms(0).elements("message").setValue("Cannot delete, this category is in use.");
            }

        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");
    }

    public ActionForward saveCategories(ActionMapping mapping,
                                        ActionForm aform,
                                        HttpServletRequest request,
                                        HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);
        try {

            CategoryForm form = (CategoryForm) aform;
            Manager.getCategoryManager(contextInfo).saveCategories(form.getCategories());

        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");


    }

}