package domlet.kbsample.web.actions;

import domlet.kbsample.data.User;
import domlet.kbsample.managers.ContextInfo;
import domlet.kbsample.managers.Manager;
import domlet.kbsample.managers.UserManager;
import domlet.kbsample.util.DbUtil;
import domlet.kbsample.web.forms.LoginForm;
import net.sf.jolene.dom.Document;
import net.sf.jolene.struts.DomletAction;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Nov 13, 2005
 * Time: 6:48:22 PM
 */
public class LoginAction extends DomletAction {

    private static final Logger log = LogManager.getLogger(LoginAction.class);


    public ActionForward login(ActionMapping mapping,
                               ActionForm aform,
                               HttpServletRequest request,
                               HttpServletResponse response) throws IOException {


        ContextInfo contextInfo = ContextInfo.getInstance(request);
        try {
            if (request.getSession().getAttribute("user") != null) {
                // User already logged in.
                return mapping.findForward("success");
            }


            LoginForm form = (LoginForm) aform;

            UserManager manager = Manager.getUserManager(contextInfo);
            User user;
            user = manager.authenticateUser(form.getUsername(), form.getPassword());
            if (user != null) {
                request.getSession().setAttribute("user", user);
                contextInfo.setUser(user);
                request.getSession().setAttribute("queryInfo", null);

                return mapping.findForward("success");
            } else {
                Document document = getDocument(mapping, request);
                if (document != null) {
                    document.forms(0).elements("login.instructions").setValue("Login failed. Try again.");
                } else {
                    log.warn("document is null in login action");
                }
                return mapping.findForward("login");
            }
        } finally {
            contextInfo.cleanUp();
        }
    }

    public ActionForward requestLogin(ActionMapping mapping,
                                      ActionForm aform,
                                      HttpServletRequest request,
                                      HttpServletResponse response) throws IOException {

        ContextInfo contextInfo = ContextInfo.getInstance(request);
        try {
            Document document = getDocument(mapping, request);
            document.forms(0).elements("login.instructions").setValue("Please log in.");
        } finally {
            contextInfo.cleanUp();
        }
        return mapping.findForward("success");

    }


    public ActionForward welcome(ActionMapping mapping,
                                 ActionForm aform,
                                 HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            DbUtil.createTables(contextInfo.getConnection(), true);
            getDocument(mapping, "success", request);

        } finally {
            contextInfo.cleanUp();
        }


        return mapping.findForward("success");
    }

    public ActionForward logout(ActionMapping mapping,
                                ActionForm aform,
                                HttpServletRequest request,
                                HttpServletResponse response) throws IOException {
        try {
            ContextInfo contextInfo = ContextInfo.getInstance(request);
            contextInfo.cleanUp();
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        request.getSession().invalidate();
        return mapping.findForward("success");
    }

}
