package domlet.kbsample.web.actions;

import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionForm;
import org.apache.log4j.Logger;
import org.apache.log4j.LogManager;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.sf.jolene.struts.DomletAction;
import net.sf.jolene.dom.Document;
import net.sf.jolene.dom.Grid;
import net.sf.jolene.dom.GridColumn;
import domlet.kbsample.managers.ContextInfo;
import domlet.kbsample.managers.Manager;
import domlet.kbsample.managers.constants.UserTypes;
import domlet.kbsample.web.viewValues.UserView;
import domlet.kbsample.web.forms.UserForm;
import domlet.kbsample.data.User;

import java.util.List;
import java.util.ArrayList;

/**
 * Created by IntelliJ IDEA.
 * User: Dan
 * Date: Oct 1, 2006
 * Time: 8:00:40 AM
 */
public class UserAction extends DomletAction {

    private static final Logger log = LogManager.getLogger(UserAction.class);

    public ActionForward browseUsers(ActionMapping mapping,
                                     ActionForm aform,
                                     HttpServletRequest request,
                                     HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            Document document = getDocument(mapping, request);
            List<UserView> list = Manager.getUserManager(contextInfo).getUserViews();

            Grid grid = (Grid) document.forms(0).elements("grid1");

            GridColumn col;
            ArrayList<GridColumn> header;
            ArrayList<String> fields;

            header = new ArrayList<GridColumn>(1);
            col = new GridColumn("User", "90%");
            header.add(col);

            fields = new ArrayList<String>(3);
            fields.add("name");
            if (contextInfo.getUser() != null && UserTypes.Admin.toString().equals(contextInfo.getUser().getTypeOfUser()))
            {
                col = new GridColumn("", "5%");
                col.setAlign("center");
                header.add(col);

                col = new GridColumn("", "5%");
                col.setAlign("center");
                header.add(col);
                fields.add("edit");
                fields.add("delete");

            } else {
                col.setWidth("100%");
                document.forms(0).elements("new").setRenderable(false);
            }

            grid.bind(header, fields, list);


        } finally {
            contextInfo.cleanUp();
        }


        return mapping.findForward("success");
    }

    public ActionForward editUser(ActionMapping mapping,
                                     ActionForm aform,
                                     HttpServletRequest request,
                                     HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            Document document = getDocument(mapping, request);
            UserForm form = (UserForm)aform;
            if (form.getId() > 0) {
                User user = Manager.getUserManager(contextInfo).getUser(form.getId());
                document.forms(0).populate(user);
                // Blank out password. We don't want to send it back.
                document.forms(0).elements("password").setValue("");
            } else {
                document.forms(0).elements("id").setValue("-1");
                document.forms(0).elements("title").setValue("New User");

            }

        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");

    }

    public ActionForward saveUser(ActionMapping mapping,
                                     ActionForm aform,
                                     HttpServletRequest request,
                                     HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {
            UserForm form = (UserForm)aform;
            User user;
            if (form.getId() <= 0) {
                user = new User();
                user.setId(0);
            } else {
                user = Manager.getUserManager(contextInfo).getUser(form.getId());
            }

            user.setAllowVisibleEmail(form.isAllowVisibleEmail());
            user.setEmail(form.getEmail());
            user.setName(form.getName());
            user.setTypeOfUser(form.getTypeOfUser());
            user.setUsername(form.getUsername());

            if (form.getPassword().length() > 0) {
                if (! form.getPassword().equalsIgnoreCase(form.getConfirm())) {
                    Document document = getDocument(mapping, request);
                    document.forms(0).populate(form);
                    document.forms(0).elements("password.error").setValue("Passwords must match");
                    return mapping.findForward("fail");
                } else {
                    user.setPassword(form.getPassword());
                }
            }
            Manager.getUserManager(contextInfo).saveUser(user);

        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");

    }

    public ActionForward deleteUser(ActionMapping mapping,
                                     ActionForm aform,
                                     HttpServletRequest request,
                                     HttpServletResponse response) throws Exception {

        ContextInfo contextInfo = ContextInfo.getInstance(request);

        try {

            Document document = getDocument(mapping, request);
            UserForm form = (UserForm)aform;
            if (Manager.getUserManager(contextInfo).isSafeToDelete(form.getId())) {
                Manager.getUserManager(contextInfo).deleteUser(form.getId());
                document.forms(0).elements("message").setValue("");
            } else {
                document.forms(0).elements("message").setValue("Cannot delete, this user is in use.");
            }

        } finally {
            contextInfo.cleanUp();
        }

        return mapping.findForward("success");

    }
}