package net.sf.jolene.dom;

/**
 * A grid column object. This is used to define the columns that the grid will display. The columns define the
 * header text, width, align and cell object. The cell object can be any of the HTMLElement objects. Cell objects
 * render the object in the grid cell instead of just text. Commonly this would be used to display checkboxes in
 * the grid.
 *
 * @author Dan Howard
 * @since Oct 11, 2005 Time: 6:51:04 PM
 */
public final class GridColumn {

    private String width;
    private String header;
    private String align;   //@todo does not work. Assigns to the header only!
    private HTMLElement cellObject;

    /**
     * Default constructor.
     */
    public GridColumn() {
        width = "";
        header = "";
        align = "";
        cellObject = null;
    }

    /**
     * Constructs a GridColumn using a header string and a width string.
     *
     * @param header header string for the column.
     * @param width  width of the column.
     */
    public GridColumn(String header, String width) {
        this();
        this.header = header;
        this.width = width;
    }

    /**
     * Return the align for the column.
     *
     * @return Alignment of the column.
     */
    public String getAlign() {
        return align;
    }


    /**
     * Gets the CellObject for the column.
     *
     * @return HTMLElement cell object.
     */
    public HTMLElement getCellObject() {
        return cellObject;
    }

    /**
     * Returns the column header string.
     *
     * @return Header of the column.
     */
    public String getHeader() {
        return header;
    }

    /**
     * Returns the column width.
     *
     * @return Width of the column.
     */
    public String getWidth() {
        return width;
    }

    /**
     * Sets the column align.
     *
     * @param align Alignment of the column.
     */
    public void setAlign(String align) {
        this.align = align;
    }

    /**
     * Sets the column header.
     *
     * @param header Header of the column.
     */
    public void setHeader(String header) {
        this.header = header;
    }

    /**
     * Sets the CellObject for the column.
     *
     * @param cellObject Any HTMLElement object.
     */
    public void setCellObject(HTMLElement cellObject) {
        this.cellObject = cellObject;
    }

    /**
     * Sets the column width.
     *
     * @param width Width of the column.
     */
    public void setWidth(String width) {
        this.width = width;
    }

}
