package net.sf.jolene.dom;

import net.sf.jolene.constants.Tags;

/**
 * An element in the header in a html document. Usually a meta, script, link or style tag.
 *
 * @author Dan Howard
 * @since Sep 19, 2006 9:11:49 AM
 */
public final class Header extends HTMLElement {

    /**
     * Default construcor.
     */
    public Header() {
        tag = Tags.meta;
    }

    /**
     * Construct a Header using a predefined Tag. Note the shoot in the footness here. You can define a header with
     * any defined Tag.  Really you should only use meta, link, script or style.
     *
     * @param tag A Tag enum. Usually script, link, meta.
     */
    public Header(Tags tag) {
        this();
        this.tag = tag;
    }


    /**
     * Returns a clone of the header element object.
     *
     * @return Header object.
     */
    @Override
    public Header clone() {
        return (Header) super.clone();
    }

    @Override
    public void setValue(String value) {
        super.setValue(value);
        setContent(value);
    }


    /**
     * Renders the Header.
     *
     * @return The header element as a html string.
     */
    @Override
    public String toString() {

        if (swapWith != null) {
            return swapWith.toString();
        }

        if (Tags.script == tag || Tags.style == tag) {
            return super.toString() + getContent() + "</" + tag + '>';
        }
        return super.toString();
    }
}
