package net.sf.jolene.dom;

import net.sf.jolene.constants.Tags;

/**
 * A select option object.
 *
 * @author Dan Howard
 * @since Nov 30, 2003
 */
public final class Option {

    int index;
    boolean selected;
    String text;
    String value;

    //Select parent;

    Tags tag = Tags.option;

    /**
     * Creates a new Option object.
     */
    public Option() {
        index = -1;
        selected = false;
        text = "";
        value = "";
    }

    /**
     * Creates an Option object using the String value and text parameters.
     *
     * @param value The value of the option
     * @param text  The display text of the option
     */
    public Option(String value, String text) {
        index = -1;
        selected = false;
        this.text = text;
        this.value = value;
    }

    /**
     * Creates an Option object using an int value and String text parameters.
     *
     * @param value The value of the option
     * @param text  The display text of the option
     */
    public Option(int value, String text) {
        index = -1;
        selected = false;
        this.text = text;
        this.value = String.valueOf(value);
    }


    /**
     * Return the index of this option in the list of selections.
     *
     * @return the index of this option in the list of selections.
     */
    public int getIndex() {
        // Read only.
        return index;
    }

    /**
     * Get the option text.
     *
     * @return text
     */
    public String getText() {
        return text;
    }

    /**
     * Get the option value.
     *
     * @return value
     */
    public String getValue() {
        return value;
    }

    /**
     * Determines if the option is currently selected. This boolean value is read/write.
     *
     * @return selected
     */
    public boolean isSelected() {
        return selected;
    }

    /**
     * Sets the option as currently selected.
     *
     * @param selected boolean
     */
    public void setSelected(boolean selected) {
        this.selected = selected;
    }

    /**
     * Sets the text to display in the option.
     *
     * @param string text to display.
     */
    public void setText(String string) {
        text = string;
    }

    /**
     * Sets the value which would be submitted when the form is submitted.
     *
     * @param string value of the option.
     */
    public void setValue(String string) {
        // Trim the value to make sure that there are no \t\n\r chars.
        value = string.trim();
    }

    /**
     * Renders the option.
     *
     * @return The option as a html string.
     */
    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer("");

        sb.append(System.getProperty("line.separator"));
        sb.append("<").append(tag).append(" value=\"").append(this.value).append("\"");

        if (selected) {
            sb.append(" selected='selected'>");
        } else {
            sb.append(">");
        }
        sb.append(this.text).append("</").append(tag).append(">");

        return new String(sb);
    }

}
