package net.sf.jolene.dom;

import net.sf.jolene.constants.Elements;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.html.Attributes;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Iterator;
import java.util.StringTokenizer;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Aug 7, 2005
 * Time: 8:20:54 AM
 */
final class Parser {

    private static final Logger log = LogManager.getLogger(Parser.class);


    String workBuffer;
    Document document;

    Parser(String workBuffer, Document document) throws MalformedHTMLException {
        this.workBuffer = workBuffer;
        this.document = document;
        parse();
    }

    void parse() throws MalformedHTMLException {

        long startTime;

        startTime = System.currentTimeMillis();

        // Set the doctype field.
        int n;
        n = workBuffer.toUpperCase().indexOf("<HTML>");
        if (n > -1) {
            // Note we trim to remove any spaces or line feeds
            document.doctype = workBuffer.substring(0, n).trim();
        }

        if (!parseHead()) {
            return;
        }

        if (!parseBody()) {
            return;
        }

        log.info("time to parse doc: " + document + " " + (System.currentTimeMillis() - startTime));
    }

    private boolean parseHead() {

        String head, uhead;
        int n1, n2;
        head = workBuffer;
        uhead = head.toUpperCase();

        n1 = uhead.indexOf("<HEAD");
        n2 = uhead.indexOf("</HEAD");
        if (n1 == -1 || n2 == -1) {
            return true; // Return true. Having no header is not critial
        }

        // Assign head to be just the head contents
        head = head.substring(n1 + 6, n2);
        uhead = head.toUpperCase();

        // See where <title> is
        int t1, t2;
        t1 = uhead.indexOf("<TITLE");
        t2 = uhead.indexOf("</TITLE");


        if (t1 == -1) {
            document.setTitle("");
        } else {
            // Retrieve the title string and then remove the title/tags.
            //System.out.println(t1);
            //System.out.println(t2);
            document.setTitle(head.substring(t1 + 7, t2));

            // Remove this from the string.
            StringBuffer s = new StringBuffer(head);
            s.delete(t1, t2 + 8);
            head = s.toString();
        }

        StringTokenizer st = new StringTokenizer(head, "<");
        String cc;
        int pos, end;

        Header scriptElement = null;
        String scriptContent = "";

        Header styleElement = null;
        String styleContent = "";

        while (st.hasMoreTokens()) {
            cc = st.nextToken();
            log.debug("RAW head: " + cc);

//            if (cc.toUpperCase().startsWith("STYLE")) {
//                log.debug("break");
//            }

            if (cc.length() > 4 && cc.toUpperCase().startsWith("META")) {
                pos = head.indexOf(cc) - 1;
                end = cc.indexOf(">");
                if (end > -1) {
                    end = pos + end + 1;
                    cc = head.substring(pos, end);
                    log.debug(cc);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in HEAD tag - position: " + pos + " " + cc);
                }

                log.debug("head tag META: " + cc);
                cc = cc.substring(5);
                Attributes a = parseAttributes(cc);
                addHeader(a, Tags.meta);

            }

            if (cc.length() > 4 && cc.toUpperCase().startsWith("LINK")) {
                pos = head.indexOf(cc) - 1;
                end = cc.indexOf(">");
                if (end > -1) {
                    end = pos + end + 1;
                    cc = head.substring(pos, end);
                    log.debug(cc);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in HEAD tag - position: " + pos + " " + cc);
                }

                log.debug("head tag LINK: " + cc);
                cc = cc.substring(5);
                Attributes a = parseAttributes(cc);
                addHeader(a, Tags.link);

            }

            // SCRIPT handling
            if (cc.length() > 6 && cc.toUpperCase().startsWith("/SCRIPT")) {
                scriptElement = null;
                scriptContent = "";
            }

            if (scriptElement != null) {
                // At this point in time we're in the middle of a script which includes a < in it's
                // expression somewhere. Append to the current substring to the current script
                // including the < character since it's not in our tokenized string.
                String script = scriptElement.getValue();
                scriptElement.setValue(script + '<' + cc);
            }

            if (cc.length() > 6 && cc.toUpperCase().startsWith("SCRIPT")) {
                pos = head.indexOf(cc) - 1;
                end = cc.indexOf(">");
                if (end > -1) {
                    //end = pos + end + 1;
                    if (cc.length() > end + 1) {
                        scriptContent = cc.substring(end + 1);
                    } else {
                        scriptContent = "";
                    }
                    cc = cc.substring(0, end);
                    log.debug(cc);
                    log.debug("script:" + scriptContent);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in HEAD tag - position: " + pos + " " + cc);
                }

                log.debug("head tag SCRIPT: " + cc + " len: " + cc.length());
                Attributes a;
                if (cc.length() > 7) {
                    cc = cc.substring(7);
                    a = parseAttributes(cc);
                } else {
                    a = new Attributes();
                }
                scriptElement = addHeader(a, Tags.script);
                scriptElement.setValue(scriptContent);
            }

            // STYLE handling
            if (cc.length() > 5 && cc.toUpperCase().startsWith("/STYLE")) {
                styleElement = null;
                styleContent = "";
            }

            if (styleElement != null) {
                // At this point in time we're in the middle of a style.
                String style = styleElement.getValue();
                styleElement.setValue(style + '<' + cc);
            }

            if (cc.length() > 5 && cc.toUpperCase().startsWith("STYLE")) {
                pos = head.indexOf(cc) - 1;
                end = cc.indexOf(">");
                if (end > -1) {
                    //end = pos + end + 1;
                    if (cc.length() > end + 1) {
                        styleContent = cc.substring(end + 1);
                    } else {
                        styleContent = "";
                    }
                    cc = cc.substring(0, end);
                    log.debug(cc);
                    log.debug("style:" + styleContent);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in HEAD tag - position: " + pos + " " + cc);
                }

                log.debug("head tag STYLE: " + cc + " len: " + cc.length());
                Attributes a;
                if (cc.length() > 6) {
                    cc = cc.substring(6);
                    a = parseAttributes(cc);
                } else {
                    a = new Attributes();
                }
                styleElement = addHeader(a, Tags.style);
                styleElement.setValue(styleContent);
            }


        }

        return true;
    }

    private Header addHeader(Attributes attributes, Tags tag) {
        Iterator i = attributes.keySet().iterator();
        Header element = new Header();
        element.tag = tag;
        String key, value;
        while (i.hasNext()) {
            key = (String) i.next();
            value = attributes.getAttribute(key);
            element.setAttribute(key, value);
        }

        document.head.add(element);
        return element;
    }

    private boolean parseBody() throws MalformedHTMLException {
        String bodyProperties;

        // Defines the body field in the class

        int n1, n2, n;
        //bodyText = workBuffer.toUpperCase();
        String c;
        c = workBuffer.toUpperCase();
        n1 = c.indexOf("<BODY");
        n2 = c.indexOf("</BODY");
        if (n1 == -1 || n2 == -1) {
            return false;
        }

        // Define the body text field in the class.
        document.bodyText = workBuffer.substring(n1 + 5, n2);

        // bodyText is all the text between body /body...
        //System.out.println(bodyText);

        // at this point trimmed(bodyText) could start with a > to indicate the end of the body tag
        // if so it means there are no properties for the body defined.

        if (!document.bodyText.trim().startsWith(">")) {
            n = document.bodyText.indexOf(">");
            bodyProperties = document.bodyText.trim().substring(0, n - 1);
            // c = the propery string for the body tag.

            log.debug("about to open body");
            document.body = parseAttributes(bodyProperties);
            document.bodyText = document.bodyText.substring(n + 1).trim();

        } else {
            document.body = new Attributes();
            document.bodyText = document.bodyText.substring(2).trim();
        }

        if (log.isDebugEnabled()) {
            BufferedWriter out = null;
            try {
                out = new BufferedWriter(new FileWriter("c:\\temp\\body.txt"));
                out.write(document.bodyText);
            } catch (IOException e) {
                log.error(e.getMessage(), e);
            } finally {
                if (out != null) {
                    try {
                        out.close();
                    } catch (IOException e) {
                    }
                }
            }

        }
        return parseForms();
    }

    private boolean parseForms() throws MalformedHTMLException {

        //System.out.println("----BODY START");
        //System.out.println(bodyText);
        //System.out.println("----BODY END");
        //		try {
        //			BufferedWriter f = new BufferedWriter( new FileWriter("body.txt") );
        //			f.write(bodyText);
        //			f.flush();
        //			f.close();
        //
        //		}
        //		catch (IOException e) {
        //
        //			e.printStackTrace();
        //		}

        // perform a search for <FORM tags and create hashMaps for the forms and
        // their controls....

        StringTokenizer st = new StringTokenizer(document.bodyText, "<");
        String cc;
        //System.out.println("ParseForms");
        int n = -1;
        int nn;
        Form f;
        int start = 0;
        int end = 0;
        boolean inForm = false, inComments = false;

        while (st.hasMoreTokens()) {
            cc = st.nextToken();

            log.debug(cc);

            if (cc.length() > 5
                    && cc.toUpperCase().substring(0, 5).equals("/FORM")) {
                inForm = false;
            }

            if (cc.length() > 4
                    && cc.toUpperCase().substring(0, 4).equals("FORM")) {

                inForm = true;

                start = document.bodyText.indexOf(cc, end) - 1;
                nn = cc.indexOf(">");
                if (nn > -1) {
                    end = start + nn + 1;
                    cc = cc.substring(0, nn);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in FORM tag - position: " + start);
                }

                //System.out.println("-->" + cc + "<--");
                //System.out.println( "" + start + " " + end);

                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.
                cc = cc.substring(4);

                f = new Form();
                f.setStartPoint(start);
                f.setEndPoint(end);
                log.debug("about to open form " + (n + 1));

                f.setAttributes(parseAttributes(cc));

                if (f.hasAttribute("name")) {
                    f.setName(f.getAttribute("name"));
                }

                document.forms.add(f);
                n++;

                //Iterator i = f.properties.keySet().iterator();

                //Iterator i2 = h.values().iterator();
                //				while (i.hasNext()) {
                //					String ccc;
                //					ccc = (String)i.next();
                //					System.out.println(ccc + " = " +  f.properties.get(ccc));
                //				}

                // At this point we have
                // docuement.forms[n] to set it's objects....

            }

            //System.out.println(cc + " " + cc.length());

            if (cc.length() > 5
                    && cc.toUpperCase().substring(0, 5).equals("INPUT") && n > -1) {
                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.
                start = document.bodyText.indexOf(cc, end) - 1;

                nn = cc.indexOf(">");
                if (nn > -1) {
                    end = start + nn + 1;
                    cc = cc.substring(0, nn);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in INPUT tag - position: " + start);
                }

                cc = cc.substring(5);

                log.debug("about to open input");

                if (inForm) {
                    Attributes a = parseAttributes(cc);
                    if (a.hasAttribute("type") && a.getAttribute("type").equalsIgnoreCase("radio")) {
                        document.forms(n).addObject(Elements.radio, a, start, end);
                    } else if (a.hasAttribute("type") && a.getAttribute("type").equalsIgnoreCase("checkbox")) {
                        document.forms(n).addObject(Elements.checkbox, a, start, end);
                    } else {
                        document.forms(n).addObject(Elements.input, a, start, end);
                    }
                }

            }

            // TODO Add support for Select OPTGROUP TAG
            if (cc.length() > 6
                    && cc.toUpperCase().substring(0, 6).equals("SELECT") && n > -1) {
                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.
                start = document.bodyText.indexOf(cc, end) - 1;
                nn = cc.indexOf(">");
                if (nn > -1) {
                    //end = start+nn+1;
                    cc = cc.substring(0, nn);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in SELECT tag - position: " + start);
                }

                end = document.bodyText.toUpperCase().indexOf("</SELECT>", start);
                if (end > -1) {
                    end += "</SELECT>".length() - 1;
                } else {
                    // MALFORMED HTML
                    throw new MalformedHTMLException("No </SELECT> found in SELECT tag - position: "
                            + start);
                }

                cc = cc.substring(6);

                log.debug("about to open select");

                Attributes a = parseAttributes(cc);
                Select selectObject = null;

                if (inForm) {
                    selectObject = (Select) document.forms(n).addObject(Elements.select, a, start, end);
                }

                if (selectObject.hasAttribute("multiple")) {
                    selectObject.setMultiple(true);
                }

                //System.out.println("---------");
                //System.out.println(selectObject);
                //System.out.println("---------");

                // Continue parsing for the options
                // Continue tokenizing along with the same tokeninzer
                // create an array list of "Option" objects
                //System.out.println("OPTIONS:");
                Option option;

                int index = -1;

                while (true) {
                    // Continue looking for option until /select
                    // Note that a malformed HTML would not be thown if there is
                    // no /select - we'd get a no more tokens instead...
                    cc = st.nextToken();
                    if (cc.toUpperCase().startsWith("/SELECT>")) {
                        break;
                    }
                    //System.out.println(cc);
                    //String ccc;
                    //ccc =cc.toUpperCase().substring(0,4);
                    //System.out.println();
                    if (cc.length() > 6
                            && cc.toUpperCase().substring(0, 6).equals("OPTION")) {

                        index++;

                        // 2 cases
                        // option value="1">ABC
                        // or
                        // option>ABC
                        option = new Option();
                        option.index = index;
                        log.debug("about to open options");

                        a = parseAttributes(cc);
                        nn = cc.toUpperCase().indexOf("VALUE");
                        if (nn > -1) {
                            nn = cc.indexOf(">");
                            option.value = a.getAttribute("VALUE");
                            option.text = cc.substring(nn + 1);
                            option.selected = a.hasAttribute("SELECTED");
                        } else {
                            nn = cc.indexOf(">");
                            option.value = cc.substring(nn + 1);
                            option.text = cc.substring(nn + 1);
                            option.selected = a.hasAttribute("SELECTED");
                        }
                        // If the select object is null then it means we're not inside a form
                        if (selectObject != null) {
                            selectObject.addOption(option);
                        }
                    }

                }

            }

            if (cc.length() > 8
                    && cc.toUpperCase().substring(0, 8).equals("TEXTAREA") && n > -1) {

                log.debug("textarea RAW:" + cc);

                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.
                start = document.bodyText.indexOf(cc, end) - 1;
                nn = cc.indexOf(">");
                String text = "";
                if (nn > -1) {
                    end = start + nn + 1;
                    text = cc.substring(nn + 1);
                    cc = cc.substring(0, nn);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in TEXTAREA tag - position: " + start);
                }

                end += text.length() + "</TEXTAREA>".length();
                cc = cc.substring(8);

                log.debug("about to open textarea");

                if (inForm) {
                    Attributes a = parseAttributes(cc);
                    TextArea o;

                    o = (TextArea) document.forms(n).addObject(Elements.textarea, a, start, end);
                    o.setValue(text);
                }

                // Move the tokenizer to the next token. Don't need to look at </texarea> tag.
                cc = st.nextToken();
            }

            if (cc.length() > 5
                    && cc.toUpperCase().substring(0, 5).equals("LABEL") && n > -1) {
                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.

                // Note: The label tag could surround other HTML which means that this
                // current token 'cc' may not contain the info required to properly
                // open this object.
                // What we have to do is use the bodyText string to grab all the text
                // in between the label /label tags and the also move the tokenizer
                // forward so it doesn't open the contents of the label.

                // At this point cc would look something like:
                // label for=whatever name=whatever>other text BUT NOT ALL OF IT...

                //System.out.println("tokeninzed for lablel");
                //System.out.println(cc);

                start = document.bodyText.indexOf(cc, end) - 1;
                nn = cc.indexOf(">");
                if (nn > -1) {
                    end = start + nn + 1;
                    cc = cc.substring(0, nn);
                    // get rid of the extra stuff after ">"
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in LABEL tag - position: " + start);
                }
                cc = cc.substring(5);
                // Get's rid of the 'label' part

                // Parse out the label attributes.
                log.debug("about to open label");

                Attributes a = parseAttributes(cc);

                // now figure out what the contents of the label is.
                int labelEnd;
                labelEnd = document.bodyText.toUpperCase().indexOf("</LABEL>", end);
                String text = "";
                text = document.bodyText.substring(end + 1, labelEnd);
                end += text.length() + "</LABEL>".length();

                // Move the tokenizer to the end of the label tokens.
                // Don't need to look at </label> tag.
                // If there are any other HTML tags then we create a Text Object,
                // otherwise we create a Label object.
                int innerTagCount = 0;
                while (!cc.toUpperCase().startsWith("/LABEL>")) {
                    cc = st.nextToken();
                    innerTagCount++;
                }

                if (inForm) {
                    HTMLElement element;
                    if (innerTagCount > 1) {
                        element = document.forms(n).addObject(Elements.text, a, start, end);
                    } else {
                        element = document.forms(n).addObject(Elements.label, a, start, end);
                    }
                    element.setValue(text);
                    //element.setText(text);
                }

            }

            if (cc.length() > 6
                    && cc.toUpperCase().substring(0, 6).equals("BUTTON") && n > -1) {
                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.

                // Note: The button tag could surround other HTML which means that this
                // current token 'cc' may not contain the info required to properly
                // open this object.
                // What we have to do is use the bodyText string to grab all the text
                // in between the button /button tags and the also move the tokenizer
                // forward so it doesn't open the contents of the button.

                // At this point cc would look something like:
                // <BUTTON NAME=B1 ID=B1>
                // Some Text (maybe HTML)
                // </BUTTON>

                start = document.bodyText.indexOf(cc, end) - 1;
                nn = cc.indexOf(">");
                if (nn > -1) {
                    end = start + nn + 1;
                    cc = cc.substring(0, nn);
                    // get rid of the extra stuff after ">"
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in BUTTON tag - position: " + start);
                }
                cc = cc.substring(6);
                // Get's rid of the 'button' part

                // Parse out the button attributes.
                log.debug("about to open button");

                Attributes a = parseAttributes(cc);

                // now figure out what the contents of the button is.
                int buttonEnd;
                buttonEnd = document.bodyText.toUpperCase().indexOf("</BUTTON>", end);
                String text = "";
                text = document.bodyText.substring(end + 1, buttonEnd);
                end += text.length() + "</BUTTON>".length();

                if (inForm) {
                    Button button = (Button) document.forms(n).addObject(Elements.button, a, start, end);
                    button.setContent(text);
                }

                // Move the tokenizer to the end of the label tokens.
                // No need to look at </button> tag.
                while (!cc.toUpperCase().startsWith("/BUTTON>")) {
                    cc = st.nextToken();
                }

            }

            if (cc.length() > 3
                    && cc.toUpperCase().substring(0, 3).equals("IMG") && n > -1) {
                // Remove the 1st word in the string. Ie. INPUT,  SELECT,  LABEL etc..
                // Not required to open.
                start = document.bodyText.indexOf(cc, end) - 1;

                nn = cc.indexOf(">");
                if (nn > -1) {
                    end = start + nn + 1;
                    cc = cc.substring(0, nn);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in IMG tag - position: " + start);
                }

                cc = cc.substring(3);

                log.debug("object: " + cc);
                log.debug("about to open img");

                if (inForm) {
                    Attributes a = parseAttributes(cc);
                    document.forms(n).addObject(Elements.image, a, start, end);
                }
            }

            // TABLE
            if (cc.length() > 5 && cc.toUpperCase().substring(0, 5).equals("TABLE") && n > -1) {
                int saveStart, saveEnd;
                saveStart = start;
                saveEnd = end;

                start = document.bodyText.indexOf(cc, end) - 1;

                nn = cc.indexOf(">");
                if (nn > -1) {
                    //end = start + nn + 1;
                    cc = cc.substring(0, nn);
                } else {
                    // Malformed HTML!!!!
                    throw new MalformedHTMLException("No > found in TABLE tag - position: " + start);
                }

                end = document.bodyText.toUpperCase().indexOf("</TABLE>", start);
                if (end > -1) {
                    end += "</TABLE>".length() - 1;
                } else {
                    // MALFORMED HTML
                    throw new MalformedHTMLException("No </TABKE> found in TABLE tag - position: "
                            + start);
                }


                cc = cc.substring(5);

                log.debug("object: " + cc);
                log.debug("about to open table");

                Attributes a = parseAttributes(cc);
                if (a.hasAttribute("id") && "GRID".equalsIgnoreCase(a.getAttribute("id").substring(0, 4)) && inForm) {
                    Grid grid;
                    grid = (Grid) document.forms(n).addObject(Elements.grid, a, start, end);
                    parseTable(grid, st);

                } else {
                    // don't add tables without ID
                    start = saveStart;
                    end = saveEnd;
                }


            }


        }
        return true;
    }

    private void parseTable(Grid grid, StringTokenizer st) {
        while (true) {
            String cc;

            // Continue looking for tr until /table
            // Note that a malformed HTML would not be thown if there is
            // no /table - we'd get a no more tokens instead...
            cc = st.nextToken();
            if (cc.toUpperCase().startsWith("/TABLE>")) {
                break;
            }

            if (cc.length() > 2
                    && cc.toUpperCase().substring(0, 2).equals("TR")) {

                cc = cc.substring(2);
                log.debug("parseTable cc: " + cc);

                GridRow row = new GridRow();

                // If cc == > then it means that TR has no attributes.
                if (cc.startsWith(">")) {
                    row.setAttributes(new Attributes());
                } else {
                    row.setAttributes(parseAttributes(cc));
                }

                while (true) {
                    cc = st.nextToken();
                    if (cc.toUpperCase().startsWith("/TR") ||
                            cc.toUpperCase().startsWith("TR") ||
                            cc.toUpperCase().startsWith("/TABLE")) {
                        break;
                    }
                    if (cc.length() > 2
                            && (cc.toUpperCase().substring(0, 2).equals("TD") || cc.toUpperCase().substring(0, 2).equals("TH"))) {

                        cc = cc.substring(2);

                        GridCell cell = new GridCell();

                        // If cc == > then it means that TR has no attributes.
                        if (cc.startsWith(">")) {
                            cell.setAttributes(new Attributes());
                        } else {
                            cell.setAttributes(parseAttributes(cc));
                        }

                        // Set the cell value
                        int n;
                        n = cc.indexOf(">");
                        if (n > -1) {
                            cell.setValue(cc.substring(n + 1));
                        }
                        row.cells.add(cell);


                    }


                }
                grid.rows.add(row);

            }

        }

    }

    private Attributes parseAttributes(String c) {
        // c is expected to be in the form of a normal HTML name/value pair.
        // This code handles extra spaces, spaces inside of quotes, etc...

        // Does not properly handle malformed (mismatched quotes)

        log.debug("RAW: " + c);


        Attributes a;
        String key, value;
        int n;
        key = "";
        value = "";

        a = new Attributes();
        if (c.trim().length() == 0) {
            // no attributes
            return a;
        }

        // remove possible extra spaces around =.
        while (c.indexOf(" =") > 0) {
            c = c.replaceAll(" =", "=");
        }

        while (c.indexOf("= ") > 0) {
            c = c.replaceAll("= ", "=");
        }

        // Remove trailing '>' if exists
        n = c.indexOf(">");
        if (n > -1) {
            c = c.substring(0, n);
        }

        // Check if right most char is a / and remove it.
        // It could be sent if the html has a form like this:
        // <img src="logo_bw.gif" alt="JetBrains logo"/>
        // instead of
        // <img src="logo_bw.gif" alt="JetBrains logo">
        if (c.charAt(c.length() - 1) == '/') {
            c = c.substring(0, c.length() - 1);
        }

        // Remove trailing \ if exists
        log.debug("string to tokenize: " + c);
        StringTokenizer st = new StringTokenizer(c);


        String cc;
        int tokenCount = st.countTokens();
        int currentToken = 0;

        while (st.hasMoreTokens()) {
            cc = st.nextToken();
            n = cc.indexOf("=");

            if (n > -1) {
                // if there's an equal, the left of it is the key
                key = cc.substring(0, n);
                value = cc.substring(n + 1);
                log.debug("token: " + cc);
                log.debug("parseAttributes: key: " + key + " value: " + value);
                currentToken = 0;
                while ((value.startsWith("\"") && !value.endsWith("\""))
                        || (value.startsWith("'") && !value.endsWith("'"))) {

                    cc = st.nextToken();
                    currentToken++;

                    value += " " + cc;

                    if (currentToken > tokenCount) {
                        // This would indicate a malformed html string. example:
                        // Left quoted but no right quote.
                        break;
                    }
                }

                if (currentToken > tokenCount) {
                    throw new MalformedHTMLException("BAD HTML string parsing attributes: " + c);
                }
            } else {
                // This would be keywords like CHECKED or MULTIPLE
                // Add =true
                key = cc;
                value = key;
            }

            if (key.length() != 0) {

                //System.out.println("Key:" + key + " value:" + value);

                // Remove " and ' chars from value
                // If the 1st char is a ' or a " remove the 1st and last char.
                if (value.startsWith("'") || value.startsWith("\"")) {
                    if (value.length() > 3) {
                        value = value.substring(1, value.length() - 1);
                    } else if (value.length() == 3) {
                        value = value.substring(1, 2);
                    } else {
                        value = "";
                    }
                }

                a.setAttribute(key, value);

            } else {

                throw new MalformedHTMLException("BAD HTML string parsing attributes - key length 0?");

            }

        }

        return a;
    }


}
