package net.sf.jolene.util;

import net.sf.jolene.constants.Prefs;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Iterator;
import java.util.Properties;

/**
 * Class to read jolene preferences from the jolene.properties file.
 *
 * @author Dan Howard
 * @since Dec 30, 2005 10:41:11 AM
 */
public class PrefsReader {

    private static final Logger log = LogManager.getLogger(PrefsReader.class);

    private PrefsReader() {
    }

    public static void init() {
        Object o = new Object();
        synchronized (o) {
            Properties properties = new Properties();
            InputStream input = null;
            try {
                URL resource = PrefsReader.class.getResource("/jolene.properties");
                if (resource == null) {
                    log.info("jolene.properties not found. Default prefs will be used");
                    return;
                }

                log.info("Reading jolene.properties from " + resource.getFile());
                input = PrefsReader.class.getResourceAsStream("/jolene.properties");
                properties.load(input);
            } catch (IOException e) {
                log.warn(e.getMessage(), e);
            }

            Iterator it = properties.keySet().iterator();
            while (it.hasNext()) {
                Object key;
                key = it.next();

                Prefs pref = Prefs.valueOf(key.toString());
                if (pref != null) {
                    log.info("Assigning jolene preference " + key + " to " + properties.get(key));
                    pref.setValue(properties.get(key));
                } else {
                    log.warn("Unknown PrefsReader preference in properties file: " + key);
                }
            }

            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    log.warn(e.getMessage());
                }
            }

        }
    }
}
