package net.sf.jolene.util;

import net.sf.jolene.dom.*;

import java.io.*;
import java.util.Properties;


/**
 * Not used.
 * User: Dan
 * Date: Jun 1, 2008
 * Time: 5:22:33 PM
 *
 * todo dumb class name - it's not translating anything.
 */
public class Translator {

    private final static PrintStream ps = System.out;

    /**
     * java -jar jolene.jar net.sf.jolene.util.Translator
     *
     * @param args 2 params are expected: full path to the FOLDER where the html files reside - full path to the output resources file.
     */
    public static void main(String args[]) {

        if (args.length < 2) {
            ps.println("Translator requires 2 parameters");
            ps.println("\t 1) full path to the FOLDER where the html files reside");
            ps.println("\t 2) full path to the output resources file");
            ps.println();
            ps.println("Example:");
            ps.println("java -jar jolene.jar d:/projects/myproject/html d:/projects/myproject/resources/application.properties");
            System.exit(0);
        }

        new Translator(args[0], args[1]).translate();
    }


    private String sourceFolder;
    private String targetPropertiesFile;
    private Properties properties;

    Translator(String sourceFolder, String targetPropertiesFile) {
        this.sourceFolder = sourceFolder;
        this.targetPropertiesFile = targetPropertiesFile;

        properties = new Properties();
        try {

            File f = new File(targetPropertiesFile);
            f.createNewFile();
            InputStream in = new FileInputStream(f);
            properties.load(in);

        } catch (IOException e) {
            ps.println(e.getMessage() + "\n" + e.getStackTrace());
            System.exit(1);
        }

    }

    private void translate() {
        File dir = new File(sourceFolder);
        ps.println("DIR: " + dir.getAbsolutePath());
        translate(dir);

        try {
            File f = new File(targetPropertiesFile);
            f.createNewFile();
            OutputStream out = new FileOutputStream(f);
            properties.store(out, "---Locale strings written by jolene---");
        } catch (IOException e) {
            ps.println(e.getMessage() + "\n" + e.getStackTrace());
            System.exit(1);
        }
    }

    private void translate(File dir) {
        File[] files = dir.listFiles();
        if (files == null) {
            ps.println("No files found in " + sourceFolder);
            return;

        } else {
            for (int i = 0; i < files.length; i++) {
                if (files[i].isDirectory()) {
                    if (!files[i].getName().equalsIgnoreCase(".svn")) {
                        translate(files[i]);
                    }
                } else {
                    //String filename = files[i].getAbsolutePath();
                    translateDocument(files[i]);
                }
            }
        }

    }

    /**
     * @param file File object of full path to a document.
     *             In order to get the message name, we use the document name which is derived from the file name with
     *             the sourceFolder path removed. Example:
     *             c:/proj/html/abc.html ==> abc
     *             c:/proj/html/invoice/invoiceEdit.html ==> invoice.invoiceEdit
     *             c:/proj/html/invoice/invoiceList.html ==> invoice.invoiceList
     *             This is to ensure uniqueness of the document names.
     * @todo generalize this - unit test this.
     * @todo we could also look for special attributes like TITLE which have text in them.
     */
    private void translateDocument(File file) {

        String filename = file.getAbsolutePath();
        String sourcefolder = new File(sourceFolder).getAbsolutePath();

        ps.println("file: " + filename);
        ps.println("sourceFolder: " + sourcefolder);

        String docname = filename.substring(sourcefolder.length() + 1);
        ps.println(docname);
        int n = docname.indexOf(".");
        if (n > -1) {
            docname = docname.substring(0, n);
        }
        ps.println(docname);
        docname = docname.replace("/", ".");
        docname = docname.replace("\\", ".");
        ps.println(docname);

        try {
            Document document = new Document(filename);
            HTMLElement element;
            StringBuilder docAndFormName;
            for (int j = 0; j < document.getFormCount(); j++) {
                docAndFormName = new StringBuilder();
                docAndFormName.append(docname).append('.');
                Form form = (Form) document.forms(j);
                if (form.getName().trim().length() > 0) {
                    docAndFormName.append(form.getName()).append('.');
                } else {
                    docAndFormName.append("form").append(j + 1).append('.');
                }

                String keyName;
                for (int k = 0; k < document.forms(j).getElementCount(); k++) {
                    element = document.forms(j).elements(k);
                    //@todo the code below often falls under the ELSE keyName = "element" + k; for some reason
                    //@todo decide - just use getName since it's ID or NAME and if empty don't create a property
/*
                    if (element.getName().trim().length() == 0) {
                        keyName = element.getAttribute("id");
                    } else {
                        keyName = element.getName();
                    }

                    if (keyName == null || keyName.trim().length() == 0) {
                        if (element.hasAttribute("type") && element.getAttribute("type").equalsIgnoreCase("submit")) {
                            keyName = "Submit";
                        }
                    } else {
                        keyName = "element" + k;
                    }
*/
                        keyName = element.getName();
                    keyName = docAndFormName + keyName;

                    if (!(element instanceof Text) && element instanceof Label) {
                        ps.println("object: " + keyName + " value: " + element.getValue());
                        if (element.getValue() != null) {
                            properties.put(keyName, element.getValue());
                        }
                    } else
                    if (element.hasAttribute("type") && (element.getAttribute("type").equalsIgnoreCase("button") || element.getAttribute("type").equalsIgnoreCase("submit"))) {
                        ps.println("object: " + keyName + " value: " + element.getValue());
                        if (element.getValue() != null) {
                            properties.put(keyName, element.getValue());
                        }
                    }
                }
            }

        } catch (IOException e) {
            ps.println(e.getMessage() + "\n" + e.getStackTrace());
            System.exit(1);
        }

    }

/*
    private void x() {
        Properties properties = new Properties();
        //@todo - so many issues here. Null pointer possibility, What value for objects? Which objects? Should they include html? NO! Wrong place for the code. Where to create the file?

        FileOutputStream out = null;
        FileInputStream in = null;
        try {
            File f = new File("resources/application.properties");
            f.createNewFile();
            in = new FileInputStream(f);
            properties.load(in);
            out = new FileOutputStream(f);

            Form ff;
            StringBuilder objectName;

            log.info("FILENAME: " + this.fileName);
            String docName;
            docName = this.fileName.substring(this.fileName.lastIndexOf("/") + 1);
            docName = docName.substring(0, docName.indexOf("."));
            log.info("DOC NAME: " + docName);
            for (int j = 0; j < forms.size(); j++) {
                objectName = new StringBuilder();
                objectName.append(docName).append('.');

                ff = (Form) forms(j);
                if (ff.name.trim().length() > 0) {
                    objectName.append(ff.name).append('.');
                } else {
                    objectName.append("form").append(j + 1).append('.');
                }

                String elementName;
                HTMLElement element;
                for (int k = 0; k < ff.elements.size(); k++) {
                    element = ff.elements(k);
                    if (element.hasAttribute("name")) {
                        elementName = element.getAttribute("name");
                    } else if (element.hasAttribute("id")) {
                        elementName = element.getAttribute("id");
                    } else {
                        elementName = element.getTag().toString() + k;
                    }
                    if (element.getValue() != null) {
                        properties.put(objectName + elementName, element.getValue());
                    }
                }
            }
            properties.store(out, "---Locale strings written by jolene---");
            out.close();
            in.close();
        } finally {
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e) {
                    log.warn(e.getMessage(), e);
                }
                try {
                    in.close();
                } catch (IOException e) {
                    log.warn(e.getMessage(), e);
                }

            }
        }

    }
*/


}
