/**
 * Comments for ${NAME} go here.
 * @author Dan Howard
 * @since Oct 7, 2008 6:45:12 PM
 */
package domlet.kbsample.data;

import junit.framework.TestCase;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.h2.jdbcx.JdbcConnectionPool;
import org.h2.jdbcx.JdbcDataSource;

import java.io.File;
import java.sql.Connection;
import java.util.List;
import java.util.Date;

import net.sf.persist.Persist;
import domlet.kbsample.util.DbUtil;
import domlet.kbsample.managers.constants.UserTypes;

public class TestH2 extends TestCase {

    private static final Logger log = LogManager.getLogger(TestH2.class);

    Connection con;
    JdbcConnectionPool cp;
    Persist persist;

    protected void setUp() throws Exception {
        super.setUp();
        Class.forName("org.h2.Driver");
        JdbcDataSource ds = new JdbcDataSource();
        ds.setURL("jdbc:h2:test");
        ds.setUser("sa");
        ds.setPassword("");
        cp = JdbcConnectionPool.create(ds);

        con = cp.getConnection();
        persist = new Persist(con);
        persist.setUpdateAutoGeneratedKeys(true);

        DbUtil.createTables(con, false);

    }

    protected void tearDown() throws Exception {
        super.tearDown();
        con.close();
        cp.dispose();

        String dir = System.getProperty("user.dir");

        String[] files = new File(dir).list();
        for (int j = 0; j < files.length; j++) {
            if (files[j].startsWith("test.")) {
                log.info("tearDown: deleting: " + dir + "/" + files[j]);
                new File(dir + "/" + files[j]).delete();
            }
        }
    }

    public void testUsers() {
        try {


            User user1 = new User();
            user1.setAllowVisibleEmail(true);
            user1.setEmail("test@.cow.com");
            user1.setName("User 1");
            user1.setTypeOfUser(UserTypes.Admin.toString());
            user1.setUsername("ABC");
            user1.setPassword("ABC");
            persist.insert(user1);

            User user2 = new User();
            user2 = new User();
            user2.setAllowVisibleEmail(true);
            user2.setEmail("test2@.cow.com");
            user2.setName("User 2");
            user2.setTypeOfUser(UserTypes.Admin.toString());
            user2.setUsername("ABC");
            user2.setPassword("ABC");
            persist.insert(user2);

            List<User> list = persist.readList(User.class,"select * from KB_USERS order by id");
            assertEquals("list size s/b 2",2,list.size());



            User user = list.get(0);
            assertEquals("user 1 should have id = 1", 1, user.getId());
            assertEquals("Type s/b Admin","Admin", user.getTypeOfUser());

            user = persist.read(User.class,"select * from KB_USERS where id = ?",1);
            assertEquals("user 1 should have id = 1", 1, user.getId());
            assertEquals("Type s/b Admin","Admin", user.getTypeOfUser());


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }


    public void testCategories() {
        try {
            Category cat1 = new Category();
            cat1.setCategory("cat 1");
            persist.insert(cat1);

            Category cat2 = new Category();
            cat2.setCategory("cat 2");
            persist.insert(cat2);

            List<Category> list = persist.readList(Category.class,"select * from KB_CATEGORIES order by id");
            assertEquals("list size s/b 2",2,list.size());

            Category cat = list.get(0);
            assertEquals("Category 1 should have 1d =1 ", 1, cat.getId());


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }

    public void testArticles() {
        try {
            Article art1 = new Article();
            art1.setActive(true);
            art1.setArticle("blah");
            art1.setCategoryId(1);
            art1.setUsers("clalala");
            art1.setCreated(new Date());
            art1.setCreatorId(1);
            art1.setModified(new Date());
            art1.setModifierId(2);
            art1.setSecurity("P");
            art1.setSubject("hello");

            persist.insert(art1);

            Article art2 = new Article();
            art2.setActive(true);
            art2.setArticle("blah");
            art2.setCategoryId(1);
            art2.setUsers("clalala");
            art2.setCreated(new Date());
            art2.setCreatorId(1);
            art2.setModified(new Date());
            art2.setModifierId(2);
            art2.setSecurity("P");
            art2.setSubject("hello");

            persist.insert(art2);

            List<Article> list = persist.readList(Article.class,"select * from KB_ARTICLES order by id");
            assertEquals("list size s/b 2",2,list.size());

            Article art = list.get(0);
            assertEquals("Article 1 should have 1d =1 ", 1, art.getId());


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }


}
