package net.sf.jolene.dom;

import junit.framework.TestCase;
import net.sf.jolene.data.Customer;
import net.sf.jolene.data.TestData;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringWriter;
import java.net.URL;
import java.util.Iterator;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Oct 12, 2005
 * Time: 5:49:53 AM
 */
public class DocumentObjectModelTest extends TestCase {

    private static final Logger log = LogManager.getLogger(DocumentObjectModelTest.class);

    public void testGetElementById() {

        Document document = null;
        URL url = getClass().getResource("/resources/TestTable.html");
        try {
            document = new Document(url.getFile());
            assertNotNull("getElementByID grid1 not null", document.elements("grid1"));
            assertNull("getElementByID grid2 is null", document.elements("grid2"));

            Grid grid = ((Grid) document.elements("grid1"));
            grid.rows.get(0).cells.get(0).setValue("COW");
            grid.rows.get(3).cells.get(2).setValue("COW2");
            grid.rows.get(3).cells.get(2).setAttribute("bgColor", "#00FF00");
            document.stream("c:\\temp\\test.html");

        } catch (FileNotFoundException e) {
            fail(e.getMessage());
        } catch (IOException e) {
            log.error(e.getMessage(), e);
        }

    }

    public void testCustomer() {
        Document document = null;
        URL url = getClass().getResource("/resources/Customer.html");
        try {

            Customer customer = new TestData().getCustomers().get(0);
            document = new Document(url.getFile());
            StringWriter sw = new StringWriter();
            document.streamResults(sw);
            assertEquals("doc content s/b ","form : 0 <form action=\"\" method=\"post\" name=\"form1\">\n" +
                    "element : 0\n" +
                    "<input id=\"name\" name=\"name\" type=\"text\">\n" +
                    "element : 1\n" +
                    "<input id=\"address\" name=\"address\" type=\"text\">\n" +
                    "element : 2\n" +
                    "<input id=\"phone\" name=\"phone\" type=\"text\">\n" +
                    "element : 3\n" +
                    "<input id=\"languageEnglish\" name=\"language\" type=\"radio\" value=\"English\">\n" +
                    "element : 4\n" +
                    "<input id=\"languageFrench\" name=\"language\" type=\"radio\" value=\"French\">\n" +
                    "element : 5\n" +
                    "<input id=\"languageOther\" name=\"language\" type=\"radio\" value=\"Other\">\n" +
                    "element : 6\n" +
                    "<input name=\"active\" type=\"checkbox\">\n" +
                    "element : 7\n" +
                    "<input name=\"Search\" onclick=\"Search()\" type=\"button\" value=\"search\">\n" +
                    "element : 8\n" +
                    "<input name=\"Categories\" onclick=\"Categories()\" type=\"button\" value=\"categories\">\n" +
                    "element : 9\n" +
                    "<input name=\"Logout\" onclick=\"Logout()\" type=\"button\" value=\"logout\">\n" +
                    "element : 10\n" +
                    "<button name=\"MyButton\" type=\"submit\" value=\"MyButton\">CLICK ME</button>\n" +
                    "form : 1 <form action=\"\">\n" +
                    "element : 0\n" +
                    "<input name=\"cow2\" type=\"button\">\n", sw.toString().replace("\r",""));

            Map m = BeanUtils.describe(customer);
            Iterator it;
            it = m.keySet().iterator();
            while (it.hasNext()) {
                HTMLElement w = null;
                String name = it.next().toString();
                w = document.forms(0).elements(name);
                log.debug("object name: " + name);
                if (w != null) {
                    log.debug("setting: " + name + " " + m.get(name));
                    w.setValue(m.get(name).toString());
                }
            }

            // now loop for radios
            Iterator<HTMLElement> it2;
            it2 = document.forms(0).elements.iterator();
            while (it2.hasNext()) {
                HTMLElement w = it2.next();
                if (w instanceof Radio) {
                    // If there is a property in the bean with the same name then
                    // compare this value to the radio value. If they match
                    // set the radio value to the widget value.
                    if (m.containsKey(w.getAttribute("name"))) {
                        if (w.getValue().equals(BeanUtils.getProperty(customer, w.getAttribute("name")))) {
                            w.setValue(BeanUtils.getProperty(customer, w.getAttribute("name")));
                        }
                    }
                }
            }
            document.stream("c:\\temp\\testCustomer.html");

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testNames() {
        //@todo test to make sure that the objects have names (if they exist).
    }

    public void testText() throws IOException {
        Document document = null;
        URL url = getClass().getResource("/resources/TestTable.html");

        document = new Document(url.getFile());

        log.info("label 1 class: " + document.forms(0).elements("label1").getClass().getName());
        log.info("label 2 class: " + document.forms(0).elements("label2").getClass().getName());

        boolean castErr = false;
        try {
            Text t = (Text) document.forms(0).elements("label1");
        } catch (ClassCastException e) {
            castErr = true;
        }
        assertTrue("label1 s/b not be Text", castErr);

        castErr = false;
        try {
            Label l = (Label) document.forms(0).elements("label1");
        } catch (ClassCastException e) {
            castErr = true;
        }
        assertFalse("lable1 s/b a Label", castErr);

        Text t = (Text) document.forms(0).elements("label2");
        Label l = (Label) document.forms(0).elements("label1");

        log.info("label: " + l);
        // <LABEL id="label1">Some text</LABEL>
        assertEquals("label 1 s/b <label id=\"label1\">Some text</label>", l.toString(), "<label id=\"label1\">Some text</label>");

        log.info("text:  " + t);
        // <b>Some text</b>
        assertEquals("Label 2 s/b <b>Some text</b>", t.toString(), "<b>Some text</b>");
    }
}
