/**
 * Comments for ${NAME} go here.
 * @author Dan Howard
 * @since Aug 23, 2008 4:47:27 PM
 */
package net.sf.jolene.dom;

import junit.framework.TestCase;
import net.sf.jolene.constants.Tags;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.StringWriter;
import java.net.URL;

public class DocumentTest extends TestCase {

    private static final Logger log = LogManager.getLogger(DocumentTest.class);

    static final String userdir = System.getProperty("user.dir");
    static final String ls = System.getProperty("line.separator");

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testAddHeader() {
        try {
            URL url = getClass().getResource("/resources/TestHeaders.html");
            Document document = new Document(url.getFile());
            Header h = new Header(Tags.meta);
            h.setAttribute("author", "Me!");
            h.setBeforeText("<!-- hello -->");
            h.setAfterText("<!-- goodbye -->");
            document.addHeader(h);

            StringWriter sw = new StringWriter();
            document.stream(sw);

            assertEquals("s/b", "<html>" + ls +
                    "<head>" + ls +
                    "<!-- hello --><meta author=\"Me!\"><!-- goodbye -->" + ls +
                    "<title>Header test</title>" + ls +
                    "</head>" + ls +
                    "<body >" + ls +
                    "<form action=\"none\" name=\"MyForm\">" + ls +
                    "    <input name=\"hello\" type=\"text\">" + ls +
                    "</form>" + ls +
                    "</body>" + ls +
                    "</html>", sw.toString());

            document.removeHeader(0);
            sw = new StringWriter();
            document.stream(sw);
            assertEquals("s/b", "<html>" + ls +
                    "<head>" + ls +
                    "<title>Header test</title>" + ls +
                    "</head>" + ls +
                    "<body >" + ls +
                    "<form action=\"none\" name=\"MyForm\">" + ls +
                    "    <input name=\"hello\" type=\"text\">" + ls +
                    "</form>" + ls +
                    "</body>" + ls +
                    "</html>", sw.toString());


        } catch (IOException e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testGetFormByName() {
        try {
            URL url = getClass().getResource("/resources/TestHeaders.html");
            Document document = new Document(url.getFile());
            Form form = document.forms("MYForm");
            assertNotNull("form should not be null", form);
            form = document.forms("COW");
            assertNull("form should be null", form);

        } catch (IOException e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testComments() {
        // Test to ensure that objects surrounded by comments are still available and that rendering still works.
        try {
            URL url = getClass().getResource("/resources/TestComments.html");
            Document document = new Document(url.getFile());

            assertNotNull("Form 2 should still be available", document.forms(1));
            assertNotNull("Elements inside comments should be available",document.forms(1).elements("hideMe"));
            assertNotNull("Elements inside comments should be available",document.forms(0).elements("save"));

            // Set some values on commented elements
            document.forms(1).elements("hideMe").setValue("HELLO!");
            document.forms(1).setAttribute("action","doSomething");
            document.forms(0).elements("save").setValue("GOODBYE!");

            StringWriter sw = new StringWriter();
            document.stream(sw);
            assertEquals("s/b","<html>" + ls +
                    "<head>" + ls +
                    "<title>Article</title>" + ls +
                    "</head>" + ls +
                    "<body >" + ls +
                    "<form action=\"saveArticle.do\" method=\"post\" name=\"articleForm\">" + ls +
                    ls +
                    "    <h1><label id=\"title\">Edit article</label></h1>" + ls +
                    "    <table width=\"100%\" border=\"0\" style=\"azimuth:behind;background:antiquewhite;background-attachment:fixed;background-color:azure;background-image:url('/resources/subfoldertest');cue-before:none;clear:inherit;empty-cells:inherit;font-family:cursive;font:xx-small;font-style:oblique;volume:medium;word-spacing:inherit;\">" + ls +
                    "        <tr>" + ls +
                    "            <td width=\"10%\"> <label id=\"labelCategory\">DO NOT SHOW THIS STRING</label></td>" + ls +
                    "            <td width=\"53%\">" + ls +
                    "                <select name=\"category\">" + ls +
                    "</select>" + ls +
                    ls +
                    "            </td>" + ls +
                    "            <td width=\"37%\">" + ls +
                    "                Created <label id=\"created\"></label>" + ls +
                    "            </td>" + ls +
                    "        </tr>" + ls +
                    "        <tr>" + ls +
                    "            <td>Subject</td>" + ls +
                    "            <td><input id=\"inputSubject\" name=\"subject\" size=\"60\" type=\"text\"></td>" + ls +
                    "            <td>" + ls +
                    "                Modified <label id=\"modified\"></label>" + ls +
                    "            </td>" + ls +
                    "        </tr>" + ls +
                    "        <tr>" + ls +
                    "            <td>Availability</td>" + ls +
                    "            <td valign=\"middle\">" + ls +
                    "                <input name=\"security\" type=\"radio\" value=\"Public\">" + ls +
                    "                Public" + ls +
                    "                <input name=\"security\" type=\"radio\" value=\"Private\">" + ls +
                    "                Private" + ls +
                    "                <input name=\"security\" type=\"radio\" value=\"Specify\">" + ls +
                    "                Specify" + ls +
                    ls +
                    "                <select multiple=\"multiple\" name=\"clients\" size=\"5\">" + ls +
                    "</select>" + ls +
                    ls +
                    "                Active?" + ls +
                    "                <input name=\"active\" type=\"checkbox\"></td>" + ls +
                    "            <td>&nbsp;</td>" + ls +
                    "        </tr>" + ls +
                    "    </table>" + ls +
                    "    <p>Article<br>" + ls +
                    "        <textarea cols=\"100\" name=\"article\" rows=\"10\"></textarea>" + ls +
                    "    </p>" + ls +
                    ls +
                    "    <p>" + ls +
                    "        <!--<input class=\"buttons\" name=\"save\" title=\"MOO MOOMOO\" type=\"submit\" value=\"GOODBYE!\">-->" + ls +
                    "        <input class=\"buttons\" name=\"close\" onclick=\"window.location='browse.do'\" title=\"meow\" type=\"button\" value=\"xClose\">" + ls +
                    "        <input name=\"id\" type=\"hidden\">" +ls +
                    "    </p>" + ls +
                    ls +
                    "    <p>&nbsp;</p>" + ls +
                    "</form>" + ls +
                    "more text 1 ..." + ls +
                    "<!--" + ls +
                    "<form action=\"doSomething\" name=\"hidethis\">" + ls +
                    "    <input name=\"hideMe\" type=\"hidden\" value=\"HELLO!\">" + ls +
                    "</form>" + ls +
                    "-->" + ls +
                    "more text 2 ..." + ls +
                    "</body>" + ls +
                    "</html>",sw.toString());

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }


    public void testBody() {
        // todo testBody
    }

}
