/**
 * Comments for ${NAME} go here.
 * @author Dan Howard
 * @since Aug 3, 2008 4:33:55 PM
 */
package net.sf.jolene.dom;

import junit.framework.TestCase;
import net.sf.jolene.data.Article;
import net.sf.jolene.util.Utils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.net.URL;
import java.util.Iterator;
import java.util.Map;

public class FormTest extends TestCase {

    private static final Logger log = LogManager.getLogger(FormTest.class);

    private Document document;
    private String lf;
    private String userdir;


    protected void setUp() throws Exception {
        super.setUp();

        URL url = getClass().getResource("/resources/TestPopulate.html");
        document = new Document(url.getFile());

        userdir = System.getProperty("user.dir");
        lf = System.getProperty("line.separator");
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testPopulate() {

        Article article = new Article();
        article.setActive(true);
        article.setArticle("Article content");
        article.setCategory(2);
        article.setClients("1,2,3,4");
        article.setCreated(Utils.getDateFromYYYYMMDDString("2008-17-14"));
        article.setId(1);
        article.setModified(Utils.getDateFromYYYYMMDDString("2008-17-15"));
        article.setSecurity("Private");
        article.setSubject("Article subject");

        document.forms(0).populate(article);


        log.info("active: " + document.forms(0).elements("active"));
        assertEquals("Active:", "<input checked=\"checked\" name=\"active\" type=\"checkbox\">", document.forms(0).elements("active").toString());

        // Testing Init selects after populate
        Select select = document.forms(0).getSelect("category");
        select.addOption(new Option(1, "Test 1"));
        select.addOption(new Option(2, "Test 2"));

        String selectText = "<select name=\"category\">" + lf +
                "<option value=\"1\">Test 1</option>" + lf +
                "<option value=\"2\" selected='selected'>Test 2</option>" + lf +
                "</select>" + lf;
        log.info("category: " + document.forms(0).elements("category"));
        assertEquals("category: ", selectText, document.forms(0).elements("category").toString());

        select = document.forms(0).getSelect("clients");
        select.addOption(new Option(1, "Test 1"));
        select.addOption(new Option(2, "Test 2"));
        select.addOption(new Option(3, "Test 3"));
        select.addOption(new Option(4, "Test 4"));
        selectText = "<select multiple=\"multiple\" name=\"clients\" size=\"5\">" + lf +
                "<option value=\"1\" selected='selected'>Test 1</option>" + lf +
                "<option value=\"2\" selected='selected'>Test 2</option>" + lf +
                "<option value=\"3\" selected='selected'>Test 3</option>" + lf +
                "<option value=\"4\" selected='selected'>Test 4</option>" + lf +
                "</select>" + lf;

        log.info("clients: " + document.forms(0).elements("clients"));
        assertEquals("clients: ", selectText, document.forms(0).elements("clients").toString());


        log.info("created: " + document.forms(0).elements("created"));
        assertEquals("created:", "<label id=\"created\">Thu May 14 00:00:00 EDT 2009</label>", document.forms(0).elements("created").toString());

        log.info("id: " + document.forms(0).elements("id"));
        assertEquals("id: ", "<input name=\"id\" type=\"hidden\" value=\"1\">", document.forms(0).elements("id").toString());

        log.info("modified: " + document.forms(0).elements("modified"));
        assertEquals("modified: ", "<label id=\"modified\">Fri May 15 00:00:00 EDT 2009</label>", document.forms(0).elements("modified").toString());

        // Tests ID and NAME in an Element.
        log.info("subject: " + document.forms(0).elements("subject"));
        assertEquals("subject: ", "<input id=\"inputSubject\" name=\"subject\" size=\"60\" type=\"text\" value=\"Article subject\">", document.forms(0).elements("subject").toString());


        log.info("CALLING getRadioGroup");
        Map<String, HTMLElement> radios = document.forms(0).getRadioGroup("security");

        Iterator<String> it = radios.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            assertEquals("key should be the value:", key, radios.get(key).getAttribute("value"));
        }
        int n = 0;
        for (HTMLElement radio : radios.values()) {

            switch (n) {
                case 0:
                    log.info("RADIO 1" + radio);
                    assertEquals("RADIO 1", "<input name=\"security\" type=\"radio\" value=\"Public\">", radio.toString());
                    break;
                case 1:
                    log.info("RADIO 2" + radio);
                    assertEquals("RADIO 2", "<input checked=\"checked\" name=\"security\" type=\"radio\" value=\"Private\">", radio.toString());
                    break;
                case 2:
                    log.info("RADIO 3" + radio);
                    assertEquals("RADIO 3", "<input name=\"security\" type=\"radio\" value=\"Specify\">", radio.toString());

            }
            n++;
        }

        radios.get("Private").setChecked(false);
        radios.get("Public").setChecked(true);
        assertEquals("Private s/b not checked","<input name=\"security\" type=\"radio\" value=\"Private\">",radios.get("Private").toString());
        assertEquals("Public s/b checked","<input checked=\"checked\" name=\"security\" type=\"radio\" value=\"Public\">",radios.get("Public").toString());

    }


}
