/**
 * User: Dan
 * Date: Jul 13, 2008
 * Time: 4:25:59 PM
 */
package net.sf.jolene.dom;

import junit.framework.TestCase;
import net.sf.jolene.data.Customer;
import net.sf.jolene.data.TestData;
import net.sf.jolene.util.Utils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.net.URL;
import java.util.ArrayList;
import java.util.List;

public class GridTest extends TestCase {

    private static final Logger log = LogManager.getLogger(GridTest.class);

    private Document document;
    private String lf;
    private String userdir;

    private Grid grid;
    private GridColumn col;
    private List<GridColumn> header;
    private List<String> fields;
    private List<Customer> data;


    @Override
    protected void setUp() throws Exception {
        super.setUp();
        URL url = getClass().getResource("/resources/TestGrid.html");
        document = new Document(url.getFile());

        userdir = System.getProperty("user.dir");
        lf = System.getProperty("line.separator");
        data = new TestData().getCustomers();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testRenderBasicGrid() {

        try {
            grid = (Grid) document.forms(0).elements("grid1");

            header = new ArrayList<GridColumn>(2);
            col = new GridColumn("ID", "");
            header.add(col);

            col = new GridColumn("Name", "");
            header.add(col);

            fields = new ArrayList<String>(2);
            fields.add("id");
            fields.add("name");

            grid.bind(header, fields, data);

            document.stream(userdir + "/test/resources/TestGrid_result.html");
            document = new Document(userdir + "/test/resources/TestGrid_result.html");

            assertNotNull("Element grid1 should be there", document.elements("grid1"));

            String grid1 = "<table class=\"Grid\" id=\"grid1\">" + lf +
                    "<tr>" + lf +
                    "\t<td>ID</td>" + lf +
                    "\t<td>Name</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "<tr class=\"OddRow\">" + lf +
                    "\t<td>1</td>" + lf +
                    "\t<td>ABC Inc</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "<tr class=\"EvenRow\">" + lf +
                    "\t<td>2</td>" + lf +
                    "\t<td>Zebra Inc</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "</table>" + lf;

            assertEquals("Grid should equal this", grid1, document.elements("grid1").toString());


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testRenderCheckboxGrid() {

        try {
            grid = (Grid) document.forms(0).elements("grid1");

            header = new ArrayList<GridColumn>(2);
            col = new GridColumn("Select", "");
            col.setCellObject(new CheckBox("selectedItems"));
            header.add(col);

            col = new GridColumn("Name", "");
            header.add(col);

            fields = new ArrayList<String>(2);
            fields.add("id");
            fields.add("name");

            grid.bind(header, fields, data);

            document.stream(userdir + "/test/resources/TestGrid_result.html");
            document = new Document(userdir + "/test/resources/TestGrid_result.html");

            assertNotNull("Element grid1 should be there", document.elements("grid1"));

            // @todo another way to test this would be to rename the grid to 'test' then when we re-parse, jolene will see the checkboxes (however, the rendering will be messed up since we don't support parsing with multiple objects with the same name).
            String content = "<html>" + lf +
                    "<head>" + lf +
                    "<title></title>" + lf +
                    "</head>" + lf +
                    "<body >" + lf +
                    "<form action=\"test\">" + lf +
                    "    <table class=\"Grid\" id=\"grid1\">" + lf +
                    "<tr>" + lf +
                    "\t<th>Select</th>" + lf +
                    "\t<th>Name</th>" + lf +
                    "</tr>" + lf +
                    lf +
                    "<tr class=\"OddRow\">" + lf +
                    "\t<td><input name=\"selectedItems\" type=\"checkbox\" value=\"1\"></td>" + lf +
                    "\t<td>ABC Inc</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "<tr class=\"EvenRow\">" + lf +
                    "\t<td><input name=\"selectedItems\" type=\"checkbox\" value=\"2\"></td>" + lf +
                    "\t<td>Zebra Inc</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "</table>" + lf +
                    lf +
                    "</form>" + lf +
                    "</body>" + lf +
                    "</html>";

            assertEquals("Content should be same", content, Utils.readFileAsString(userdir + "/test/resources/TestGrid_result.html"));


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testRenderSelectGrid() {

        try {
            grid = (Grid) document.forms(0).elements("grid1");

            header = new ArrayList<GridColumn>(2);
            col = new GridColumn("Select", "");
            Select select = new Select("title");
            select.addOption(new Option("1", "Mr"));
            select.addOption(new Option("2", "Mrs"));
            col.setCellObject(select);
            header.add(col);

            col = new GridColumn("Name", "");
            header.add(col);

            fields = new ArrayList<String>(2);
            fields.add("id");
            fields.add("name");

            grid.bind(header, fields, data);

            document.stream(userdir + "/test/resources/TestGrid_result.html");
            document = new Document(userdir + "/test/resources/TestGrid_result.html");

            assertNotNull("Element grid1 should be there", document.elements("grid1"));

            String content = "<html>" + lf +
                    "<head>" + lf +
                    "<title></title>" + lf +
                    "</head>" + lf +
                    "<body >" + lf +
                    "<form action=\"test\">" + lf +
                    "    <table class=\"Grid\" id=\"grid1\">" + lf +
                    "<tr>" + lf +
                    "\t<th>Select</th>" + lf +
                    "\t<th>Name</th>" + lf +
                    "</tr>" + lf +
                    lf +
                    "<tr class=\"OddRow\">" + lf +
                    "\t<td><select name=\"title\">" + lf +
                    "<option value=\"1\" selected='selected'>Mr</option>" + lf +
                    "<option value=\"2\">Mrs</option>" + lf +
                    "</select>" + lf +
                    "</td>" + lf +
                    "\t<td>ABC Inc</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "<tr class=\"EvenRow\">" + lf +
                    "\t<td><select name=\"title\">" + lf +
                    "<option value=\"1\">Mr</option>" + lf +
                    "<option value=\"2\" selected='selected'>Mrs</option>" + lf +
                    "</select>" + lf +
                    "</td>" + lf +
                    "\t<td>Zebra Inc</td>" + lf +
                    "</tr>" + lf +
                    lf +
                    "</table>" + lf +
                    lf +
                    "</form>" + lf +
                    "</body>" + lf +
                    "</html>";

            assertEquals("Content should be same", content, Utils.readFileAsString(userdir + "/test/resources/TestGrid_result.html"));


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    //@todo add tests for text area, input type text, Others?


}
