/**
 * Created by IntelliJ IDEA.
 * User: DanH
 * Date: Jan 8, 2006
 * Time: 6:34:50 PM
 */
package net.sf.jolene.dom;

import junit.framework.TestCase;
import net.sf.jolene.constants.Prefs;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.util.Utils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.net.URL;
import java.util.Collection;

public class HTMLObjectsTest extends TestCase {

    private static final Logger log = LogManager.getLogger(HTMLObjectsTest.class);

    static final String userdir = System.getProperty("user.dir");
    static final String ls = System.getProperty("line.separator");

    @Override
    protected void setUp() throws Exception {
        super.setUp();
    }

    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testSelectSetValue() {
        try {
            // Test to see if we can set a select value *before* defining the options.
            // It should still allow it and work.

            Document document = null;
            URL url = getClass().getResource("/resources/checkout.html");
            document = new Document(url.getFile());
            Select salutation = document.forms(1).getSelect("SALUTATION");
            salutation.clearOptions();
            salutation.setValue("Mr");

            // Should log a WARN since we're trying to get a value from the select
            // and no options are defined yet.
            salutation.getValue();

            salutation.addOption(new Option("Mr", "Mr"));
            salutation.addOption(new Option("Mrs", "Mrs"));
            salutation.addOption(new Option("Ms", "Ms"));
            salutation.addOption(new Option("Dr", "Dr"));

            for (int j = 0; j < salutation.options.size(); j++) {
                log.info(salutation.options(j).toString().trim());
                if (j == 0) {
                    log.info("Option 0 s/b selected even though value was set before.");
                    assertEquals("Option 0 s/b selected even though value was set before.", salutation.options(j).toString().trim(), "<option value=\"Mr\" selected='selected'>Mr</option>");
                }
            }

            // Try the same with name value pairs & multi select
            salutation.clearOptions();
            String sep = Prefs.SelectValueSeperator.getValue();

            salutation.setValue("1" + sep + "3");
            // Should log a WARN since we're trying to get a value from the select
            // and no options are defined yet.
            salutation.getValue();
            salutation.addOption(new Option(1, "Mr"));
            salutation.addOption(new Option(2, "Mrs"));
            salutation.addOption(new Option(3, "Ms"));
            salutation.addOption(new Option(4, "Dr"));

            log.info(salutation.getValue());

            assertTrue("Option 0 s/b selected even though value was set before.", salutation.options(0).isSelected());

            //log.info(salutation.options(2).toString().trim());
            assertTrue("Option 2 s/b selected even though value was set before.", salutation.options(2).isSelected());
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }

    public void testButton() {
        try {
            Document document = null;
            URL url = getClass().getResource("/resources/Customer.html");
            document = new Document(url.getFile());
            log.info("Search button: " + document.forms(0).elements("Search"));

            assertEquals("Seach button should have all attributes from file (sorted):", "<input name=\"Search\" onclick=\"Search()\" type=\"button\" value=\"search\">", document.forms(0).elements("Search").toString());

            HTMLElement element;
            element = document.forms(0).elements("MyButton");
            assertEquals("Button Value should be MyButton", "MyButton", element.getValue());
            assertEquals("Button TEXT should be CLICK ME", "CLICK ME", element.getContent());
            assertEquals("Button should be '<button name=\"MyButton\" type=\"submit\" value=\"MyButton\">CLICK ME</button>'", "<button name=\"MyButton\" type=\"submit\" value=\"MyButton\">CLICK ME</button>", element.toString());

            Button button = new Button("MyButton");
            button.setValue("Submit");
            Image img = new Image();
            img.setAttribute("src", "/img/smile.png");
            img.setAttribute("alt", "Smile");
            button.setContent(img + "click me");
            assertEquals("Button should be '<button name=\"MyButton\" value=\"Submit\"><img alt=\"Smile\" src=\"/img/smile.png\">click me</button>'", "<button name=\"MyButton\" value=\"Submit\"><img alt=\"Smile\" src=\"/img/smile.png\">click me</button>", button.toString());

            button = new Button();
            assertEquals("Empty button s/b '<button></button>'", "<button></button>", button.toString());

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }


    public void testSwap() {
        try {
            URL url = getClass().getResource("/resources/TestSwap.html");
            Document document;
            document = new Document(url.getFile());

            Button button = new Button();
            button.setValue("hello");
            button.setAttribute("name", "testButton");

            document.elements("testSwap").swapWith(button.toString());

            document.stream(userdir + "/test/resources/TestSwap_result.html");
            document = new Document(userdir + "/test/resources/TestSwap_result.html");

            assertNull("Element testSwap should not be there", document.elements("testSwap"));
            assertNotNull("Element testButton should be there", document.elements("testButton"));
            assertTrue("Element should be a button ", document.elements("testButton") instanceof Button);

            TextArea textarea = new TextArea();
            textarea.setValue("moo moo moo");
            textarea.setAttribute("name", "testTextArea");
            document.elements("testButton").swapWith(textarea.toString());

            document.stream(userdir + "/test/resources/TestSwap_result.html");
            document = new Document(userdir + "/test/resources/TestSwap_result.html");

            assertNull("Element testButton should not be there", document.elements("testButton"));
            assertNotNull("Element testTextArea should be there", document.elements("testTextArea"));
            assertTrue("Element should be a TextArea ", document.elements("testTextArea") instanceof TextArea);

            document.elements("testTextArea").swapWith("HELLO WORLD");
            document.stream(userdir + "/test/resources/TestSwap_result.html");

            assertEquals("Content should be same", "<html>" + ls +
                    "<head>" + ls +
                    "<title></title>" + ls +
                    "</head>" + ls +
                    "<body >" + ls +
                    "<form action=\"test\">" + ls +
                    "    HELLO WORLD" + ls +
                    "</form>" + ls +
                    "</body>" + ls +
                    "</html>", Utils.readFileAsString(userdir + "/test/resources/TestSwap_result.html"));

        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }

    public void testCheckBox() {
        try {
            Document document = null;
            URL url = getClass().getResource("/resources/TestBooleanAttributes.html");
            document = new Document(url.getFile());

            document.elements("input").setReadonly(true);
            document.elements("input").setDisabled(true);
            document.elements("checkbox").setChecked(true);
            document.forms(0).getSelect("select").setMultiple(true);

            document.stream(userdir + "/test/resources/TestBooleanAttributes_result.html");

            String content = "<html>" + ls +
                    "<head>" + ls +
                    "<title></title>" + ls +
                    "</head>" + ls +
                    "<body >" + ls +
                    "<form action=\"test\">" + ls +
                    "    <input disabled=\"disabled\" name=\"input\" readonly=\"readonly\" type=\"text\">" + ls +
                    "    <input checked=\"checked\" name=\"checkbox\" type=\"checkbox\">" + ls +
                    "    <select multiple=\"multiple\" name=\"select\">" + ls +
                    "</select>" + ls +
                    ls +
                    "</form>" + ls +
                    "</body>" + ls +
                    "</html>";

            assertEquals("Content should be same", content, Utils.readFileAsString(userdir + "/test/resources/TestBooleanAttributes_result.html"));

            document = new Document(userdir + "/test/resources/TestBooleanAttributes_result.html");
            assertTrue("Checkbox is checked", document.elements("checkbox").isChecked());

            document.elements("checkbox").setChecked(false);

            document.stream(userdir + "/test/resources/TestBooleanAttributes_result2.html");
            document = new Document(userdir + "/test/resources/TestBooleanAttributes_result2.html");

            assertFalse("Checkbox is NOT checked", document.elements("checkbox").isChecked());

            // Test new instance.
            CheckBox checkbox = new CheckBox("MOO");
            checkbox.setChecked(true);
            assertEquals("Checkbox should be '<input checked=\"checked\" name=\"MOO\" type=\"checkbox\">'", "<input checked=\"checked\" name=\"MOO\" type=\"checkbox\">", checkbox.toString());

            checkbox = new CheckBox();
            assertEquals("checkbox s/b/ '<input type=\"checkbox\">'", "<input type=\"checkbox\">", checkbox.toString());

            checkbox.setName("Cow");
            assertEquals("checkbox s/b/ '<input name=\"Cow\" type=\"checkbox\">'", "<input name=\"Cow\" type=\"checkbox\">", checkbox.toString());


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testHeader() {
        Header plain = new Header();

        assertEquals("Plain header s/b '<meta>'", "<meta>", plain.toString());

        Header meta = new Header(Tags.meta);
        meta.setAttribute("http-equiv", "Content-Type");
        meta.setAttribute("content", "text/html; charset=iso-8859-1");
        assertEquals("header s/b '<meta content=\"text/html; charset=iso-8859-1\" http-equiv=\"Content-Type\">'", "<meta content=\"text/html; charset=iso-8859-1\" http-equiv=\"Content-Type\">", meta.toString());

        Header linkStyle = new Header(Tags.link);
        linkStyle.setAttribute("href", "/css/mystyle.css");
        linkStyle.setAttribute("rel", "stylesheet");
        linkStyle.setAttribute("type", "text/css");
        assertEquals("Style s/b '<link href=\"/css/mystyle.css\" rel=\"stylesheet\" type=\"text/css\">'", "<link href=\"/css/mystyle.css\" rel=\"stylesheet\" type=\"text/css\">", linkStyle.toString());

        Header inlineStyle = new Header(Tags.style);
        inlineStyle.setAttribute("media", "SCREEN");
        inlineStyle.setContent("a { background:aliceblue }");
        assertEquals("inline style s/b '<style media=\"SCREEN\">a { background:aliceblue }</style>'", "<style media=\"SCREEN\">a { background:aliceblue }</style>", inlineStyle.toString());

        // Test setValue s/b same as setContent
        Header inlineStyle2 = new Header(Tags.style);
        inlineStyle2.setAttribute("media", "SCREEN");
        inlineStyle2.setValue("a { background:aliceblue }");
        assertEquals("inline2 style s/b '<style media=\"SCREEN\">a { background:aliceblue }</style>'", "<style media=\"SCREEN\">a { background:aliceblue }</style>", inlineStyle2.toString());

        try {
            URL url = getClass().getResource("/resources/TestHeader.html");
            Document document = new Document(url.getFile());
            Collection<Header> headers = document.head;
           
            assertEquals("list s/b 5", 5, headers.size());

            int i = 0;
            String headerStr;
            for (Header header : headers) {
                // We don't care about win vs unix line feeds.
                headerStr = header.toString().replace("\r\n", "\n");

                switch (i) {
                    case 0:
                        assertEquals("header s/b '<meta content=\"text/html; charset=iso-8859-1\" http-equiv=\"Content-Type\">'", "<meta content=\"text/html; charset=iso-8859-1\" http-equiv=\"Content-Type\">", header.toString());
                        break;
                    case 1:
                        assertEquals("header s/b '<link href=\"mystyle.css\" rel=\"stylesheet\" type=\"text/css\">'", "<link href=\"mystyle.css\" rel=\"stylesheet\" type=\"text/css\">", header.toString());
                        break;
                    case 2:

                        assertEquals("header s/b '<style media=\"screen\">\n" +
                                "        a {\n" +
                                "            background:aliceblue;\n" +
                                "        }\n" +
                                "    </style>'", "<style media=\"screen\">\n" +
                                "        a {\n" +
                                "            background:aliceblue;\n" +
                                "        }\n" +
                                "    </style>", headerStr);
                        break;
                    case 3:
                        assertEquals("header s/b '<script language=\"JavaScript\" src=\"myjs.js\" type=\"text/javascript\"></script>'", "<script language=\"JavaScript\" src=\"myjs.js\" type=\"text/javascript\"></script>", header.toString());
                        break;
                    case 4:
                        assertEquals("header s/b '<script language=\"JavaScript\" type=\"text/javascript\">\n" +
                                "        function hello() {\n" +
                                "            alert(\"Hello\");\n" +
                                "            alert(\"<form>\");\n" +
                                "            x = \"<input type=\\\"text\\\">\";\n" +
                                "            alert(\"</form>\");\n" +
                                "        }\n" +
                                "    </script>'", "<script language=\"JavaScript\" type=\"text/javascript\">\n" +
                                "        function hello() {\n" +
                                "            alert(\"Hello\");\n" +
                                "            alert(\"<form>\");\n" +
                                "            x = \"<input type=\\\"text\\\">\";\n" +
                                "            alert(\"</form>\");\n" +
                                "        }\n" +
                                "    </script>", headerStr);
                        break;
                }
                i++;

            }

            document.stream(userdir + "/test/resources/TestHeader_result.html");

            String content = Utils.readFileAsString(userdir + "/test/resources/TestHeader_result.html");
            content = content.replace("\r\n", "\n");

            assertEquals("Document should be the same", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n" +
                    "<html>\n" +
                    "<head>\n" +
                    "<meta content=\"text/html; charset=iso-8859-1\" http-equiv=\"Content-Type\">\n" +
                    "<link href=\"mystyle.css\" rel=\"stylesheet\" type=\"text/css\">\n" +
                    "<style media=\"screen\">\n" +
                    "        a {\n" +
                    "            background:aliceblue;\n" +
                    "        }\n" +
                    "    </style>\n" +
                    "<script language=\"JavaScript\" src=\"myjs.js\" type=\"text/javascript\"></script>\n" +
                    "<script language=\"JavaScript\" type=\"text/javascript\">\n" +
                    "        function hello() {\n" +
                    "            alert(\"Hello\");\n" +
                    "            alert(\"<form>\");\n" +
                    "            x = \"<input type=\\\"text\\\">\";\n" +
                    "            alert(\"</form>\");\n" +
                    "        }\n" +
                    "    </script>\n" +
                    "<title>Header test</title>\n" +
                    "</head>\n" +
                    "<body >\n" +
                    "<form>\n" +
                    "</form>\n" +
                    "</body>\n" +
                    "</html>", content);


        } catch (IOException e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }
    }

    public void testImage() {
        Image image = new Image();
        assertEquals("Empty image s/b '<img>'", "<img>", image.toString());

        image.setContent("?");
        image.setValue("??");
        assertEquals("setText and setValue should make not change '<img>'", "<img>", image.toString());

        image.setAttribute("src", "someimg.png");
        assertEquals("with source '<img src=\"someimg.png\">'", "<img src=\"someimg.png\">", image.toString());

    }

    public void testInput() {
        Input input = new Input();
        assertEquals("Empty input s/b '<input>'", "<input>", input.toString());

        input = new Input("moo");
        assertEquals("Empty input s/b '<input name=\"moo\">'", "<input name=\"moo\">", input.toString());

        input.setContent("wtf?");
        assertEquals("setText should have no effect. '<input name=\"moo\">'", "<input name=\"moo\">", input.toString());

        input.setValue("wtf?");
        assertEquals("input should have a valkue. '<input name=\"moo\" value=\"wtf?\">'", "<input name=\"moo\" value=\"wtf?\">", input.toString());
    }

    public void testLabel() {

        Label label = new Label();
        assertEquals("Empty label s/b '<label></label>'", "<label></label>", label.toString());

        label.setContent("COW");
        assertEquals("label text s/b '<label>COW</label>'", "<label>COW</label>", label.toString());

        label = new Label();
        label.setValue("COW2");
        assertEquals("label value s/b '<label>COW2</label>'", "<label>COW2</label>", label.toString());

    }

    public void testRadio() {
        Radio radio = new Radio();
        assertEquals("Empty radio s/b '<input type=\"radio\">'", "<input type=\"radio\">", radio.toString());

        radio = new Radio("cow");
        assertEquals("radio with name s/b '<input name=\"cow\" type=\"radio\">'", "<input name=\"cow\" type=\"radio\">", radio.toString());
    }

    public void testSelect() {
        Select select = new Select();
        assertEquals("Empty select s/b '<select>" + ls + "</select>" + ls + "'", "<select>" + ls + "</select>" + ls, select.toString());

        select = new Select("cow");
        assertEquals("select with name s/b '<select name=\"cow\">" + ls + "</select>" + ls + "'", "<select name=\"cow\">" + ls + "</select>" + ls, select.toString());
    }

    public void testText() {
        Text text = new Text();
        assertEquals("Empty text s/b ''", "", text.toString());

        text.setValue("HELLO");
        assertEquals("text s/b 'HELLO'", "HELLO", text.toString());

        text = new Text();
        text.setContent("HELLO2");
        assertEquals("text s/b 'HELLO2'", "HELLO2", text.toString());
    }

    public void testTextArea() {
        TextArea textArea = new TextArea();
        assertEquals("Empty TextArea s/b '<textarea></textarea>'", "<textarea></textarea>", textArea.toString());

        textArea = new TextArea("moo");
        assertEquals("TextArea with name s/b '<textarea name=\"moo\"></textarea>'", "<textarea name=\"moo\"></textarea>", textArea.toString());

        textArea.setContent("cow");
        assertEquals("TextArea with name s/b '<textarea name=\"moo\">cow</textarea>'", "<textarea name=\"moo\">cow</textarea>", textArea.toString());

        textArea = new TextArea("moo2");
        textArea.setValue("cow2");
        assertEquals("TextArea with name s/b '<textarea name=\"moo2\">cow2</textarea>'", "<textarea name=\"moo2\">cow2</textarea>", textArea.toString());

    }

    public void testStyles() {
        try {
            URL url = getClass().getResource("/resources/TestStyles.html");
            Document document = new Document(url.getFile());

            HTMLElement element = document.elements("testStyle");
            assertNotNull("element testStyle should be found", element);

            log.info("Style direct   : " + element.getStyles().toString());
            log.info("Style attribute: " + element.getAttribute("style"));

            assertEquals("direct vs from attribute s/b same ", element.getStyles().toString(), element.getAttribute("style"));

            // Test setStyle on html element
            element.setStyle("color", "pink");
            log.info("Style attribute after: " + element.getAttribute("style"));
            assertEquals("After should have color:pink", "azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;border-top-width:medium;clip:auto;color:pink;", element.getAttribute("style"));

            // Test setStyle on html attribute
            element.setAttribute("style", "color:blue");
            assertEquals("After should have color:blue", "azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;border-top-width:medium;clip:auto;color:blue;", element.getAttribute("style"));

            document.stream(userdir + "/test/resources/TestStyles_result.html");


        } catch (Exception e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }

}
