package net.sf.jolene.dom;

import junit.framework.TestCase;

import java.net.URL;
import java.util.ArrayList;
import java.util.Arrays;
import java.io.StringWriter;
import java.io.IOException;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import net.sf.jolene.data.TestData;
import net.sf.jolene.constants.Tags;
import net.sf.jolene.html.Styles;

/**
 * User: DanH
 * Date: Aug 1, 2005
 * Time: 6:57:46 PM
 */
public class ParserTest extends TestCase {

    private static final Logger log = LogManager.getLogger(ParserTest.class);


    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testForSpaceInAttributeValue() {

        long n = System.currentTimeMillis();


        URL url = getClass().getResource("/resources/testForSpaceInAttributeValue.html");

        try {
            Document document = new Document(url.getFile());
            document.stream("c:\\temp\\test.html");


        } catch (Exception e) {
            log.error(e.getMessage(), e);

            fail(e.getMessage());
        }

        log.info("time: " + (System.currentTimeMillis() - n));


    }

    public void testUnicodeParse() {
        long n = System.currentTimeMillis();


        URL url = getClass().getResource("/resources/checkout.html");

        try {
            Document document = new Document(url.getFile());
            document.stream("c:\\temp\\test.html");
            StringWriter sw = new StringWriter();
            document.streamResults(sw);
            assertEquals("content s/b","form : 0 <form action=\"/esales/checkout.html?sessionid=251974325&random=cc633ee2ac176f1fec190d2152bef3a9&sessionid=251974325&random=cc633ee2ac176f1fec190d2152bef3a9\" method=\"post\">\n" +
                    "element : 0\n" +
                    "<select name=\"languageid\" onchange=\"submit();\">\n" +
                    "<option value=\"1\">English</option>\n" +
                    "<option value=\"2\">Deutsch</option>\n" +
                    "<option value=\"3\">Português</option>\n" +
                    "<option value=\"4\">Español</option>\n" +
                    "<option value=\"5\">Italiano</option>\n" +
                    "<option value=\"6\">Français</option>\n" +
                    "<option value=\"7\">Norsk</option>\n" +
                    "<option value=\"8\">Nederlands</option>\n" +
                    "<option value=\"9\">Svenska</option>\n" +
                    "<option value=\"10\">Suomi</option>\n" +
                    "<option value=\"14\">Dansk</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 1\n" +
                    "<input type=\"submit\" value=\"Go!\">\n" +
                    "form : 1 <form action=\"checkout.html?sessionid=251974325&random=cc633ee2ac176f1fec190d2152bef3a9&prno=1\" method=\"post\" name=\"personal\">\n" +
                    "element : 0\n" +
                    "<input class=\"e5_hidden_input\" height=\"1\" name=\"fix\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" type=\"image\" value=\"1\" width=\"1\">\n" +
                    "element : 1\n" +
                    "<img border=\"0\" height=\"8\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 2\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 3\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 4\n" +
                    "<input maxlength=\"150\" name=\"RE_USERNAME\" size=\"20\" type=\"TEXT\" value=\"\">\n" +
                    "element : 5\n" +
                    "<input maxlength=\"50\" name=\"RE_PASSWORD\" size=\"20\" type=\"PASSWORD\" value=\"\">\n" +
                    "element : 6\n" +
                    "<input alt=\"Login\" name=\"BUTTON_RETURNING_C\" onclick=\"\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/english/def02/b_login.gif\" title=\"Login\" type=\"image\" value=\"Login\">\n" +
                    "element : 7\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 8\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 9\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 10\n" +
                    "<img border=\"0\" height=\"4\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 11\n" +
                    "<img border=\"0\" height=\"9\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 12\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 13\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 14\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 15\n" +
                    "<input maxlength=\"100\" name=\"EMAIL\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 16\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 17\n" +
                    "<input maxlength=\"100\" name=\"EMAIL_CONFIRM\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 18\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"200\">\n" +
                    "element : 19\n" +
                    "<input checked=\"checked\" name=\"REG_NAME_RADIO\" type=\"RADIO\" value=\"NAME\">\n" +
                    "element : 20\n" +
                    "<input name=\"REG_NAME_RADIO\" type=\"RADIO\" value=\"COMPANY\">\n" +
                    "element : 21\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"200\">\n" +
                    "element : 22\n" +
                    "<input maxlength=\"50\" name=\"COMPANY\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 23\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 24\n" +
                    "<select name=\"SALUTATION\">\n" +
                    "<option value=\"\"></option>\n" +
                    "<option value=\"MR.\">Mr.</option>\n" +
                    "<option value=\"MS.\">Ms.</option>\n" +
                    "<option value=\"MRS\">Mrs.</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 25\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 26\n" +
                    "<input maxlength=\"50\" name=\"FIRSTNAME\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 27\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 28\n" +
                    "<input maxlength=\"30\" name=\"LASTNAME\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 29\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 30\n" +
                    "<input maxlength=\"100\" name=\"D_STREET1\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 31\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 32\n" +
                    "<input maxlength=\"100\" name=\"D_STREET2\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 33\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 34\n" +
                    "<input maxlength=\"50\" name=\"D_CITY\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 35\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 36\n" +
                    "<select name=\"D_STATE_ID\">\n" +
                    "<option value=\"\"></option>\n" +
                    "<option value=\"\">-------  USA -------</option>\n" +
                    "<option value=\"al\">Alabama</option>\n" +
                    "<option value=\"ak\">Alaska</option>\n" +
                    "<option value=\"az\">Arizona</option>\n" +
                    "<option value=\"ar\">Arkansas</option>\n" +
                    "<option value=\"aa\">Armed Forces Americas (exc.Canada)</option>\n" +
                    "<option value=\"ae\">Armed Forces Eur, ME, Afr, CDN</option>\n" +
                    "<option value=\"ap\">Armed Forces Pacific</option>\n" +
                    "<option value=\"ca\">California</option>\n" +
                    "<option value=\"co\">Colorado</option>\n" +
                    "<option value=\"ct\">Connecticut</option>\n" +
                    "<option value=\"dc\">D.C.</option>\n" +
                    "<option value=\"de\">Delaware</option>\n" +
                    "<option value=\"fl\">Florida</option>\n" +
                    "<option value=\"ga\">Georgia</option>\n" +
                    "<option value=\"hi\">Hawaii</option>\n" +
                    "<option value=\"id\">Idaho</option>\n" +
                    "<option value=\"il\">Illinois</option>\n" +
                    "<option value=\"in\">Indiana</option>\n" +
                    "<option value=\"ia\">Iowa</option>\n" +
                    "<option value=\"ks\">Kansas</option>\n" +
                    "<option value=\"ky\">Kentucky</option>\n" +
                    "<option value=\"la\">Louisiana</option>\n" +
                    "<option value=\"me\">Maine</option>\n" +
                    "<option value=\"md\">Maryland</option>\n" +
                    "<option value=\"ma\">Massachusetts</option>\n" +
                    "<option value=\"mi\">Michigan</option>\n" +
                    "<option value=\"mn\">Minnesota</option>\n" +
                    "<option value=\"ms\">Mississippi</option>\n" +
                    "<option value=\"mo\">Missouri</option>\n" +
                    "<option value=\"mt\">Montana</option>\n" +
                    "<option value=\"ne\">Nebraska</option>\n" +
                    "<option value=\"nv\">Nevada</option>\n" +
                    "<option value=\"nh\">New Hampshire</option>\n" +
                    "<option value=\"nj\">New Jersey</option>\n" +
                    "<option value=\"nm\">New Mexico</option>\n" +
                    "<option value=\"ny\">New York</option>\n" +
                    "<option value=\"nc\">North Carolina</option>\n" +
                    "<option value=\"nd\">North Dakota</option>\n" +
                    "<option value=\"oh\">Ohio</option>\n" +
                    "<option value=\"ok\">Oklahoma</option>\n" +
                    "<option value=\"or\">Oregon</option>\n" +
                    "<option value=\"pa\">Pennsylvania</option>\n" +
                    "<option value=\"ri\">Rhode Island</option>\n" +
                    "<option value=\"sc\">South Carolina</option>\n" +
                    "<option value=\"sd\">South Dakota</option>\n" +
                    "<option value=\"tn\">Tennessee</option>\n" +
                    "<option value=\"tx\">Texas</option>\n" +
                    "<option value=\"ut\">Utah</option>\n" +
                    "<option value=\"vt\">Vermont</option>\n" +
                    "<option value=\"va\">Virginia</option>\n" +
                    "<option value=\"wa\">Washington (State)</option>\n" +
                    "<option value=\"wv\">West Virginia</option>\n" +
                    "<option value=\"wi\">Wisconsin</option>\n" +
                    "<option value=\"wy\">Wyoming</option>\n" +
                    "<option value=\"\">-------  Canada -------</option>\n" +
                    "<option value=\"ab\">Alberta</option>\n" +
                    "<option value=\"bc\">British Columbia</option>\n" +
                    "<option value=\"mb\">Manitoba</option>\n" +
                    "<option value=\"nb\">New Brunswick</option>\n" +
                    "<option value=\"nl\">Newfoundland and Labrador</option>\n" +
                    "<option value=\"nt\">Northwest Territories\n" +
                    "</option>\n" +
                    "<option value=\"ns\">Nova Scotia</option>\n" +
                    "<option value=\"nu\">Nunavut</option>\n" +
                    "<option value=\"on\">Ontario</option>\n" +
                    "<option value=\"pe\">Prince Edward Isl.</option>\n" +
                    "<option value=\"qc\">Quebec</option>\n" +
                    "<option value=\"sk\">Saskatchewan</option>\n" +
                    "<option value=\"yt\">Yukon Territory</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 37\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 38\n" +
                    "<input maxlength=\"50\" name=\"D_POSTALCODE\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 39\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 40\n" +
                    "<select name=\"D_COUNTRY_ID\">\n" +
                    "<option value=\"\">&ndash;&ndash;&gt; Please choose</option>\n" +
                    "<option value=\"800\">Australia</option>\n" +
                    "<option value=\"404\">Canada</option>\n" +
                    "<option value=\"1\">France</option>\n" +
                    "<option value=\"4\">Germany</option>\n" +
                    "<option value=\"3\">Netherlands</option>\n" +
                    "<option value=\"39\">Switzerland</option>\n" +
                    "<option value=\"6\">United Kingdom</option>\n" +
                    "<option value=\"400\">USA</option>\n" +
                    "<option value=\"\">------- All countries -------</option>\n" +
                    "<option value=\"660\">Afghanistan</option>\n" +
                    "<option value=\"33\">Åland Islands</option>\n" +
                    "<option value=\"70\">Albania</option>\n" +
                    "<option value=\"208\">Algeria</option>\n" +
                    "<option value=\"810\">American Samoa</option>\n" +
                    "<option value=\"43\">Andorra</option>\n" +
                    "<option value=\"330\">Angola</option>\n" +
                    "<option value=\"446\">Anguilla</option>\n" +
                    "<option value=\"891\">Antarctica</option>\n" +
                    "<option value=\"459\">Antigua and Barbuda</option>\n" +
                    "<option value=\"528\">Argentina</option>\n" +
                    "<option value=\"77\">Armenia</option>\n" +
                    "<option value=\"474\">Aruba</option>\n" +
                    "<option value=\"800\">Australia</option>\n" +
                    "<option value=\"38\">Austria</option>\n" +
                    "<option value=\"78\">Azerbaijan</option>\n" +
                    "<option value=\"453\">Bahamas</option>\n" +
                    "<option value=\"640\">Bahrain</option>\n" +
                    "<option value=\"666\">Bangladesh</option>\n" +
                    "<option value=\"469\">Barbados</option>\n" +
                    "<option value=\"73\">Belarus</option>\n" +
                    "<option value=\"102\">Belgium</option>\n" +
                    "<option value=\"421\">Belize</option>\n" +
                    "<option value=\"284\">Benin</option>\n" +
                    "<option value=\"413\">Bermuda</option>\n" +
                    "<option value=\"675\">Bhutan</option>\n" +
                    "<option value=\"516\">Bolivia</option>\n" +
                    "<option value=\"93\">Bosnia and Herzegovina</option>\n" +
                    "<option value=\"391\">Botswana</option>\n" +
                    "<option value=\"892\">Bouvet Island</option>\n" +
                    "<option value=\"508\">Brazil</option>\n" +
                    "<option value=\"357\">British Indian Ocean Territory</option>\n" +
                    "<option value=\"703\">Brunei Darussalam</option>\n" +
                    "<option value=\"68\">Bulgaria</option>\n" +
                    "<option value=\"236\">Burkina Faso</option>\n" +
                    "<option value=\"328\">Burundi</option>\n" +
                    "<option value=\"696\">Cambodia</option>\n" +
                    "<option value=\"302\">Cameroon</option>\n" +
                    "<option value=\"404\">Canada</option>\n" +
                    "<option value=\"21\">Canary Islands</option>\n" +
                    "<option value=\"247\">Cape Verde</option>\n" +
                    "<option value=\"463\">Cayman Islands</option>\n" +
                    "<option value=\"306\">Central African Republic</option>\n" +
                    "<option value=\"22\">Ceuta</option>\n" +
                    "<option value=\"244\">Chad</option>\n" +
                    "<option value=\"512\">Chile</option>\n" +
                    "<option value=\"720\">China</option>\n" +
                    "<option value=\"834\">Christmas Island</option>\n" +
                    "<option value=\"833\">Cocos (Keeling) Islands</option>\n" +
                    "<option value=\"480\">Colombia</option>\n" +
                    "<option value=\"375\">Comoros</option>\n" +
                    "<option value=\"322\">Congo, Dem. Republic</option>\n" +
                    "<option value=\"318\">Congo, Republic</option>\n" +
                    "<option value=\"837\">Cook Islands</option>\n" +
                    "<option value=\"436\">Costa Rica</option>\n" +
                    "<option value=\"272\">Cote d'Ivoire</option>\n" +
                    "<option value=\"92\">Croatia</option>\n" +
                    "<option value=\"600\">Cyprus</option>\n" +
                    "<option value=\"601\">Cyprus (unregulated area)</option>\n" +
                    "<option value=\"61\">Czech Republic</option>\n" +
                    "<option value=\"8\">Denmark</option>\n" +
                    "<option value=\"338\">Djibouti</option>\n" +
                    "<option value=\"460\">Dominica</option>\n" +
                    "<option value=\"456\">Dominican Republic</option>\n" +
                    "<option value=\"626\">East-Timor</option>\n" +
                    "<option value=\"500\">Ecuador</option>\n" +
                    "<option value=\"220\">Egypt</option>\n" +
                    "<option value=\"428\">El Salvador</option>\n" +
                    "<option value=\"310\">Equatorial Guinea</option>\n" +
                    "<option value=\"336\">Eritrea</option>\n" +
                    "<option value=\"53\">Estonia</option>\n" +
                    "<option value=\"334\">Ethiopia</option>\n" +
                    "<option value=\"529\">Falkland Islands</option>\n" +
                    "<option value=\"25\">Faroe Islands</option>\n" +
                    "<option value=\"815\">Fiji</option>\n" +
                    "<option value=\"32\">Finland</option>\n" +
                    "<option value=\"1\">France</option>\n" +
                    "<option value=\"496\">French Guiana</option>\n" +
                    "<option value=\"822\">French Polynesia</option>\n" +
                    "<option value=\"894\">French Southern Territories</option>\n" +
                    "<option value=\"314\">Gabon</option>\n" +
                    "<option value=\"252\">Gambia</option>\n" +
                    "<option value=\"76\">Georgia</option>\n" +
                    "<option value=\"4\">Germany</option>\n" +
                    "<option value=\"276\">Ghana</option>\n" +
                    "<option value=\"44\">Gibraltar</option>\n" +
                    "<option value=\"9\">Greece</option>\n" +
                    "<option value=\"406\">Greenland</option>\n" +
                    "<option value=\"473\">Grenada</option>\n" +
                    "<option value=\"458\">Guadeloupe</option>\n" +
                    "<option value=\"831\">Guam</option>\n" +
                    "<option value=\"416\">Guatemala</option>\n" +
                    "<option value=\"87\">Guernsey</option>\n" +
                    "<option value=\"260\">Guinea</option>\n" +
                    "<option value=\"257\">Guinea-Bissau</option>\n" +
                    "<option value=\"488\">Guyana</option>\n" +
                    "<option value=\"452\">Haiti</option>\n" +
                    "<option value=\"835\">Heard Island and McDonald Islands</option>\n" +
                    "<option value=\"424\">Honduras</option>\n" +
                    "<option value=\"740\">Hong Kong</option>\n" +
                    "<option value=\"64\">Hungary</option>\n" +
                    "<option value=\"24\">Iceland</option>\n" +
                    "<option value=\"664\">India</option>\n" +
                    "<option value=\"700\">Indonesia</option>\n" +
                    "<option value=\"7\">Ireland</option>\n" +
                    "<option value=\"86\">Isle Of Man</option>\n" +
                    "<option value=\"624\">Israel</option>\n" +
                    "<option value=\"5\">Italy</option>\n" +
                    "<option value=\"464\">Jamaica</option>\n" +
                    "<option value=\"732\">Japan</option>\n" +
                    "<option value=\"88\">Jersey</option>\n" +
                    "<option value=\"628\">Jordan</option>\n" +
                    "<option value=\"79\">Kazakhstan</option>\n" +
                    "<option value=\"346\">Kenya</option>\n" +
                    "<option value=\"812\">Kiribati</option>\n" +
                    "<option value=\"728\">Korea, South</option>\n" +
                    "<option value=\"636\">Kuwait</option>\n" +
                    "<option value=\"83\">Kyrgyzstan</option>\n" +
                    "<option value=\"684\">Laos</option>\n" +
                    "<option value=\"54\">Latvia</option>\n" +
                    "<option value=\"604\">Lebanon</option>\n" +
                    "<option value=\"395\">Lesotho</option>\n" +
                    "<option value=\"268\">Liberia</option>\n" +
                    "<option value=\"37\">Liechtenstein</option>\n" +
                    "<option value=\"55\">Lithuania</option>\n" +
                    "<option value=\"104\">Luxembourg</option>\n" +
                    "<option value=\"743\">Macau</option>\n" +
                    "<option value=\"96\">Macedonia (Former Yugoslav Republic)</option>\n" +
                    "<option value=\"370\">Madagascar</option>\n" +
                    "<option value=\"386\">Malawi</option>\n" +
                    "<option value=\"701\">Malaysia</option>\n" +
                    "<option value=\"667\">Maldives</option>\n" +
                    "<option value=\"232\">Mali</option>\n" +
                    "<option value=\"46\">Malta</option>\n" +
                    "<option value=\"824\">Marshall Islands</option>\n" +
                    "<option value=\"462\">Martinique</option>\n" +
                    "<option value=\"228\">Mauritania</option>\n" +
                    "<option value=\"373\">Mauritius</option>\n" +
                    "<option value=\"377\">Mayotte</option>\n" +
                    "<option value=\"23\">Melilla</option>\n" +
                    "<option value=\"412\">Mexico</option>\n" +
                    "<option value=\"823\">Micronesia</option>\n" +
                    "<option value=\"74\">Moldova</option>\n" +
                    "<option value=\"95\">Monaco</option>\n" +
                    "<option value=\"716\">Mongolia</option>\n" +
                    "<option value=\"470\">Montserrat</option>\n" +
                    "<option value=\"204\">Morocco</option>\n" +
                    "<option value=\"366\">Mozambique</option>\n" +
                    "<option value=\"676\">Myanmar</option>\n" +
                    "<option value=\"389\">Namibia</option>\n" +
                    "<option value=\"803\">Nauru</option>\n" +
                    "<option value=\"672\">Nepal</option>\n" +
                    "<option value=\"3\">Netherlands</option>\n" +
                    "<option value=\"478\">Netherlands Antilles</option>\n" +
                    "<option value=\"809\">New Caledonia</option>\n" +
                    "<option value=\"804\">New Zealand</option>\n" +
                    "<option value=\"432\">Nicaragua</option>\n" +
                    "<option value=\"240\">Niger</option>\n" +
                    "<option value=\"288\">Nigeria</option>\n" +
                    "<option value=\"838\">Niue</option>\n" +
                    "<option value=\"836\">Norfolk Island</option>\n" +
                    "<option value=\"820\">Northern Mariana Islands</option>\n" +
                    "<option value=\"28\">Norway</option>\n" +
                    "<option value=\"649\">Oman</option>\n" +
                    "<option value=\"662\">Pakistan</option>\n" +
                    "<option value=\"825\">Palau</option>\n" +
                    "<option value=\"625\">Palestinian Territory</option>\n" +
                    "<option value=\"442\">Panama</option>\n" +
                    "<option value=\"801\">Papua New Guinea</option>\n" +
                    "<option value=\"520\">Paraguay</option>\n" +
                    "<option value=\"504\">Peru</option>\n" +
                    "<option value=\"708\">Philippines</option>\n" +
                    "<option value=\"813\">Pitcairn Islands</option>\n" +
                    "<option value=\"60\">Poland</option>\n" +
                    "<option value=\"10\">Portugal</option>\n" +
                    "<option value=\"444\">Puerto Rico</option>\n" +
                    "<option value=\"644\">Qatar</option>\n" +
                    "<option value=\"372\">Reunion</option>\n" +
                    "<option value=\"66\">Romania</option>\n" +
                    "<option value=\"75\">Russia</option>\n" +
                    "<option value=\"324\">Rwanda</option>\n" +
                    "<option value=\"819\">Samoa</option>\n" +
                    "<option value=\"47\">San Marino</option>\n" +
                    "<option value=\"311\">Sao Tome and Principe</option>\n" +
                    "<option value=\"632\">Saudi Arabia</option>\n" +
                    "<option value=\"248\">Senegal</option>\n" +
                    "<option value=\"97\">Serbia and Montenegro</option>\n" +
                    "<option value=\"355\">Seychelles</option>\n" +
                    "<option value=\"264\">Sierra Leone</option>\n" +
                    "<option value=\"706\">Singapore</option>\n" +
                    "<option value=\"63\">Slovakia</option>\n" +
                    "<option value=\"91\">Slovenia</option>\n" +
                    "<option value=\"806\">Solomon Islands</option>\n" +
                    "<option value=\"342\">Somalia</option>\n" +
                    "<option value=\"388\">South Africa</option>\n" +
                    "<option value=\"893\">South Georgia and Sandwich Islands</option>\n" +
                    "<option value=\"11\">Spain</option>\n" +
                    "<option value=\"669\">Sri Lanka</option>\n" +
                    "<option value=\"329\">St. Helena</option>\n" +
                    "<option value=\"449\">St. Kitts and Nevis</option>\n" +
                    "<option value=\"465\">St. Lucia</option>\n" +
                    "<option value=\"408\">St. Pierre and Miquelon</option>\n" +
                    "<option value=\"467\">St. Vincent and the Grenadines</option>\n" +
                    "<option value=\"492\">Suriname</option>\n" +
                    "<option value=\"27\">Svalbard</option>\n" +
                    "<option value=\"393\">Swaziland</option>\n" +
                    "<option value=\"30\">Sweden</option>\n" +
                    "<option value=\"39\">Switzerland</option>\n" +
                    "<option value=\"736\">Taiwan</option>\n" +
                    "<option value=\"82\">Tajikistan</option>\n" +
                    "<option value=\"352\">Tanzania</option>\n" +
                    "<option value=\"680\">Thailand</option>\n" +
                    "<option value=\"280\">Togo</option>\n" +
                    "<option value=\"839\">Tokelau</option>\n" +
                    "<option value=\"817\">Tonga</option>\n" +
                    "<option value=\"472\">Trinidad and Tobago</option>\n" +
                    "<option value=\"212\">Tunisia</option>\n" +
                    "<option value=\"52\">Turkey</option>\n" +
                    "<option value=\"80\">Turkmenistan</option>\n" +
                    "<option value=\"454\">Turks and Caicos Islands</option>\n" +
                    "<option value=\"807\">Tuvalu</option>\n" +
                    "<option value=\"350\">Uganda</option>\n" +
                    "<option value=\"72\">Ukraine</option>\n" +
                    "<option value=\"647\">United Arab Emirates</option>\n" +
                    "<option value=\"6\">United Kingdom</option>\n" +
                    "<option value=\"524\">Uruguay</option>\n" +
                    "<option value=\"400\">USA</option>\n" +
                    "<option value=\"832\">US Minor Outlying Islands</option>\n" +
                    "<option value=\"81\">Uzbekistan</option>\n" +
                    "<option value=\"816\">Vanuatu</option>\n" +
                    "<option value=\"45\">Vatican City State</option>\n" +
                    "<option value=\"484\">Venezuela</option>\n" +
                    "<option value=\"690\">Viet Nam</option>\n" +
                    "<option value=\"468\">Virgin Islands (British)</option>\n" +
                    "<option value=\"457\">Virgin Islands (U.S.)</option>\n" +
                    "<option value=\"811\">Wallis and Futuna</option>\n" +
                    "<option value=\"201\">Western Sahara</option>\n" +
                    "<option value=\"653\">Yemen</option>\n" +
                    "<option value=\"378\">Zambia</option>\n" +
                    "<option value=\"382\">Zimbabwe</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 41\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 42\n" +
                    "<input maxlength=\"50\" name=\"PHONE\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 43\n" +
                    "<img border=\"0\" height=\"1\" src='https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\"width=\"200'>\n" +
                    "element : 44\n" +
                    "<input maxlength=\"50\" name=\"FAX\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 45\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"200\">\n" +
                    "element : 46\n" +
                    "<input maxlength=\"40\" name=\"VATID\" size=\"30\" type=\"TEXT\" value=\"\">\n" +
                    "element : 47\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 48\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 49\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 50\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 51\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"200\">\n" +
                    "element : 52\n" +
                    "<select name=\"PAYMENTTYPE_ID\">\n" +
                    "<option value=\"\">&ndash;&ndash;&gt; Please choose</option>\n" +
                    "<option value=\"CCA\">Credit Card</option>\n" +
                    "<option value=\"DBC\">Solo/Switch/Maestro (only if issued in UK)</option>\n" +
                    "<option value=\"WTR\">Bank/Wire Transfer</option>\n" +
                    "<option value=\"CHK\">Check</option>\n" +
                    "<option value=\"CAS\">Cash - Prepayment</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 53\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"200\">\n" +
                    "element : 54\n" +
                    "<select name=\"CURRENCY_ID\">\n" +
                    "<option value=\"\">&ndash;&ndash;&gt; Please choose</option>\n" +
                    "<option value=\"USD\">US Dollar</option>\n" +
                    "<option value=\"EUR\">Euro</option>\n" +
                    "<option value=\"AUD\">Australian Dollar</option>\n" +
                    "<option value=\"CAD\">Canadian Dollar</option>\n" +
                    "<option value=\"JPY\">Japanese Yen</option>\n" +
                    "<option value=\"GBP\">Pound Sterling</option>\n" +
                    "<option value=\"CHF\">Swiss Franc</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 55\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"200\">\n" +
                    "element : 56\n" +
                    "<input maxlength=\"25\" name=\"COUPON1\" size=\"20\" type=\"TEXT\" value=\"\">\n" +
                    "element : 57\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 58\n" +
                    "<img border=\"0\" height=\"1\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/p_t.gif\" width=\"1\">\n" +
                    "element : 59\n" +
                    "<input alt=\"Next\" name=\"B1\" onclick=\"\" src=\"https://a248.e.akamai.net/f/248/5462/2d/images.element5.com/esales/images/english/def02/b_nextstep.gif\" title=\"Next\" type=\"image\" value=\"Next\">\n" +
                    "element : 60\n" +
                    "<input name=\"progress\" type=\"hidden\" value=\"ADDITIONAL\">\n" +
                    "element : 61\n" +
                    "<input name=\"FROM_PERSONAL\" type=\"hidden\" value=\"1\">\n", sw.toString().replace("\r",""));


        } catch (Exception e) {
            log.error(e.getMessage(), e);

            fail(e.getMessage());
        }

        log.info("time: " + (System.currentTimeMillis() - n));

    }

    public void testUnicodeMess() {
        long n = System.currentTimeMillis();


        URL url = getClass().getResource("/resources/languages.html");

        try {
            Document document = new Document(url.getFile());
            document.stream("c:\\temp\\test.html");
            StringWriter sw = new StringWriter();
            document.streamResults(sw);
            assertEquals("content s/b","form : 0 <form action=\"/esales/checkout.html?sessionid=251974325&random=cc633ee2ac176f1fec190d2152bef3a9&sessionid=251974325&random=cc633ee2ac176f1fec190d2152bef3a9\" method=\"post\">\n" +
                    "element : 0\n" +
                    "<select name=\"languageid\" onchange=\"submit();\">\n" +
                    "<option value=\"1\">English</option>\n" +
                    "<option value=\"2\">Deutsch</option>\n" +
                    "<option value=\"3\">Português</option>\n" +
                    "<option value=\"4\">Español</option>\n" +
                    "<option value=\"5\">Italiano</option>\n" +
                    "<option value=\"6\">Français</option>\n" +
                    "<option value=\"7\">Norsk</option>\n" +
                    "<option value=\"8\">Nederlands</option>\n" +
                    "<option value=\"9\">Svenska</option>\n" +
                    "<option value=\"10\">Suomi</option>\n" +
                    "<option value=\"14\">Dansk</option>\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 1\n" +
                    "<select name=\"languageid2\" onchange=\"submit();\">\n" +
                    "<option value=\"1\">English</option>\n" +
                    "<option value=\"2\">Deutsch</option>\n" +
                    "<option value=\"3\">Português</option>\n" +
                    "<option value=\"4\">Español</option>\n" +
                    "<option value=\"5\">Italiano</option>\n" +
                    "<option value=\"6\">Français</option>\n" +
                    "<option value=\"7\">Norsk</option>\n" +
                    "<option value=\"8\">Nederlands</option>\n" +
                    "<option value=\"9\">Svenska</option>\n" +
                    "<option value=\"10\">Suomi</option>\n" +
                    "<option value=\"14\">Dansk</option>\n" +
                    "</select>\n\n" , sw.toString().replace("\r",""));


        } catch (Exception e) {
            log.error(e.getMessage(), e);

            fail(e.getMessage());
        }

        log.info("time: " + (System.currentTimeMillis() - n));


    }

    public void testPage() {
// D:\Java\Projects\bookmork\web\register.htm
        try {
            Document document = new Document("D:\\Java\\Projects\\bookmork\\web\\register.htm");
            document.stream("c:\\temp\\test.html");
            StringWriter sw = new StringWriter();
            document.streamResults(sw);
            assertEquals("content s/b","form : 0 <form action=\"/cgi-bin/register.cgi\" method=\"POST\">\n" +
                    "element : 0\n" +
                    "<label name=\"status\">Please fill in all the fields.</label>\n" +
                    "element : 1\n" +
                    "<input name=\"un\" size=\"20\" type=\"text\">\n" +
                    "element : 2\n" +
                    "<input name=\"pw\" size=\"20\" type=\"password\">\n" +
                    "element : 3\n" +
                    "<input name=\"first_name\" size=\"30\" type=\"text\">\n" +
                    "element : 4\n" +
                    "<input name=\"last_name\" size=\"30\" type=\"text\">\n" +
                    "element : 5\n" +
                    "<input name=\"city\" size=\"20\" type=\"text\">\n" +
                    "element : 6\n" +
                    "<input name=\"province\" size=\"20\" type=\"text\">\n" +
                    "element : 7\n" +
                    "<select name=\"country\" size=\"1\">\n" +
                    "</select>\n" +
                    "\n" +
                    "element : 8\n" +
                    "<input name=\"email\" size=\"30\" type=\"text\">\n" +
                    "element : 9\n" +
                    "<textarea a=\"10\" b=\"20\">\n" +
                    "hello\n" +
                    "</textarea>\n" +
                    "element : 10\n" +
                    "<input name=\"remember\" type=\"checkbox\">\n" +
                    "element : 11\n" +
                    "<img border=\"0\" height=\"30\" name=\"nextstep\" speedtip=\"Click to register.\" src=\"/images/nextstep1.gif\" width=\"214\">\n" +
                    "element : 12\n" +
                    "<input name=\"id\" type=\"hidden\" value=\"-1\">\n", sw.toString().replace("\r",""));


        } catch (Exception e) {
            log.error(e.getMessage(), e);

            fail(e.getMessage());
        }

    }

    public void testTable() {

        Document document = null;
        URL url = getClass().getResource("/resources/TestTable.html");
        try {
            document = new Document(url.getFile());
            Grid grid = (Grid) document.forms(0).elements("grid1");
            if (grid != null) {
                ArrayList<GridColumn> headers = new ArrayList<GridColumn>();
                //headers.add(new GridColumn());
                headers.add(new GridColumn("Customer", "40%"));
                headers.add(new GridColumn("Address", "40%"));
                headers.add(new GridColumn("Phone", "20%"));

                ArrayList<String> fields = new ArrayList<String>();
                //fields.add("Id");
                fields.add("name");
                fields.add("address");
                fields.add("phone");

                grid.bind(headers, fields, new TestData().getCustomers());

            }


            document.stream("c:\\temp\\test.html");
            StringWriter sw = new StringWriter();
            document.streamResults(sw);
            assertEquals("content s/b","form : 0 <form action=\"\" method=\"POST\">\n" +
                    "element : 0\n" +
                    "<table bgcolor=\"#0000FF\" border=\"1\" bordercolor=\"#FF0000\" class=\"Grid\" frame=\"box\" id=\"grid1\" width=\"100%\">\n" +
                    "<tr>\n" +
                    "\t<th width=\"40%\">Customer</th>\n" +
                    "\t<th width=\"40%\">Address</th>\n" +
                    "\t<th width=\"20%\">Phone</th>\n" +
                    "</tr>\n" +
                    "\n" +
                    "<tr class=\"OddRow\">\n" +
                    "\t<td width=\"40%\">ABC Inc</td>\n" +
                    "\t<td width=\"40%\">123 Sesame Street</td>\n" +
                    "\t<td width=\"20%\">123-4567</td>\n" +
                    "</tr>\n" +
                    "\n" +
                    "<tr class=\"EvenRow\">\n" +
                    "\t<td width=\"40%\">Zebra Inc</td>\n" +
                    "\t<td width=\"40%\">456 Cow Street</td>\n" +
                    "\t<td width=\"20%\">878-1234</td>\n" +
                    "</tr>\n" +
                    "\n" +
                    "</table>\n" +
                    "\n" +
                    "element : 1\n" +
                    "<input name=\"T1\" size=\"20\" type=\"text\">\n" +
                    "element : 2\n" +
                    "<input name=\"T2\" size=\"30\" type=\"text\">\n" +
                    "element : 3\n" +
                    "<label id=\"label1\">Some text</label>\n" +
                    "element : 4\n" +
                    "<b>Some text</b>\n", sw.toString().replace("\r",""));


        } catch (Exception e) {

            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }

    public void testStrings() {

        assertTrue("SELECT string", "select".equals(Tags.select.toString()));
        assertTrue("SELECT type  ", Tags.select.equals(Tags.valueOf("select")));

        log.info("</label>".substring(1));

        //fail("cow");

        String uri = "/jolene/test/login.html";

        log.info("before: " + uri);

        int n = uri.indexOf('/', 1);

        log.info("after: " + uri.substring(n + 1));

        uri = "/kbsample/jolene/test/login.html";

        log.info("before: " + uri);

        n = uri.indexOf('/', 1);

        log.info("after: " + uri.substring(n + 1));

        //fail("coo");
    }


    public void testTranslatePaths() {
        //@todo src comes back blank see readme.txt

        long n = System.currentTimeMillis();


        URL url = getClass().getResource("/resources/testForSpaceInAttributeValue.html");

        String contextPath = "/testContextPath";
        try {
            Document document = new Document(url.getFile(), contextPath,"");
            document.stream("c:\\temp\\test.html");


            HTMLElement element;
            String src;
            for (int j = 0; j < document.forms.size(); j++) {
                for (int k = 0; k < document.forms(j).elements.size(); k++) {
                    if (document.forms(j).elements(k).getTag() == Tags.img) {
                        element = document.forms(j).elements(k);
                        src = element.getAttribute("src");
                        log.info(element);
                        log.info(src);
                        assertTrue("images should start with http or contextpath", src.startsWith(contextPath) || src.toLowerCase().startsWith("http"));
                    }
                }
            }


        } catch (Exception e) {
            log.error(e.getMessage(), e);

            fail(e.getMessage());
        }

        log.info("time: " + (System.currentTimeMillis() - n));

    }

    public void testParsePageWithNoAttributes() throws Exception {
        URL url = getClass().getResource("/resources/TestNoAttributes.html");
        Document document = new Document(url.getFile());

        // TODO ADD ASSERTS
        document.stream("c:/temp/TestNoAttributes.html");
    }

    public void testParseFieldPropertyForGrid() {
        // ${module.jolene.basedir}
        String prop = "indexedCategory(${propertyName})";

        int n1 = prop.indexOf("${");
        int n2 = prop.indexOf("}");

        String propertyName = prop.substring(n1+2,n2).trim();
        log.info("propertyName: " + propertyName);
        assertEquals("property name s/b 'propertyName'","propertyName", propertyName);

    }

    public void testDocumentWithDupedElements() {
        URL url = getClass().getResource("/resources/TestDupedElements.html");
        try {
            Document document = new Document(url.getFile());

            StringWriter sw = new StringWriter();
            document.streamResults(sw);
            log.info(sw.toString());

            StringWriter sw2 = new StringWriter();
            document.stream(sw2);
            log.info(sw2.toString());

        } catch (IOException e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }

    public void testParseStyles() {

        //String styleString = "azimuth:behind;background:antiquewhite;background-attachment:fixed;background-color:azure;background-image:url('/resources/subfoldertest');\"";

        String styleString = "border-top:2px solid gray;height:expression( document.body.clientHeight - getTopPosition(document.all.resultsFrame))";



        log.info(styleString);

        log.info(Arrays.asList(styleString.split(";")));

        Styles styles = new Styles();

        String astyles[] = styleString.split(";");
        for (int j =0; j<astyles.length;j++) {
            String astyle[] = astyles[j].split(":");
            log.info("style: " + astyles[j]);
            // style could have 1 if string ends with ;
//            assertEquals("should only have 2 elements",2,astyle.length);
            if (astyle.length > 1) {
                styles.setStyle(astyle[0],astyle[1]);
            }
        }

        log.info(styles);
    }

    // todo do this for each parsed element.
    public void testMalFormedForm() {
        URL url = getClass().getResource("/resources/BadHTMForm.html");

        try {
            Document document = new Document(url.getFile());
        } catch (MalformedHTMLException e) {
            assertTrue("Malformed form html tag startswith 'No > found in FORM tag'", e.getMessage().startsWith("No > found in FORM tag"));
            log.warn(e.getMessage());

        } catch (IOException e) {
            log.error(e.getMessage(), e);
            fail(e.getMessage());
        }

    }
}
