/**
 * Comments for ${NAME} go here.
 * @author Dan Howard
 * @since Aug 5, 2008 8:45:33 PM
 */
package net.sf.jolene.html;

import junit.framework.TestCase;
import net.sf.jolene.dom.Button;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;

public class StylesTest extends TestCase {

    private static final Logger log = LogManager.getLogger(StylesTest.class);

    protected void setUp() throws Exception {
        super.setUp();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testStyles() {

        String styleString = "azimuth:behind;background:aliceblue;background-color:aquamarine;border-bottom-style:solid;clip:auto;border-top-width:medium;font:xx-small;";

        Styles styles = new Styles(styleString);

        assertEquals("size should be 7", 7, styles.keySet().size());

        assertNull("Should not be found 'poo'", styles.getStyle("poo"));

        assertNotNull("Should be found case sensitive'", styles.getStyle("background-color"));

        assertNotNull("Should be found case insensitive'", styles.getStyle("BackGrounD-CoLoR"));

        styles = new Styles("font:xx-small;azimuth:behind;background:aliceblue;");
        assertEquals("syles should be sorted.", "azimuth:behind;background:aliceblue;font:xx-small;", styles.toString());

        // setStyles should add to the styles.
        assertEquals("size should be 3", 3, styles.keySet().size());
        styles.setStyles("background-color:aquamarine;");
        assertEquals("size should be 4", 4, styles.keySet().size());
        assertEquals("syles should have new value.", "azimuth:behind;background:aliceblue;background-color:aquamarine;font:xx-small;", styles.toString());

        // Reset
        styles.resetStyles("background-color:aquamarine;");
        assertEquals("size should be 1", 1, styles.keySet().size());
        assertEquals("syles should have one value.", "background-color:aquamarine;", styles.toString());

        //setStyle (single)
        styles.setStyle("color", "blue");
        assertEquals("size should be 2", 2, styles.keySet().size());
        assertEquals("syles should have 2 values.", "background-color:aquamarine;color:blue;", styles.toString());

        assertTrue("style should exist", styles.hasStyle("color"));
        assertFalse("style should not exist", styles.hasStyle("COW"));

        styles.removeStyle("color");
        assertEquals("syles should not have color now.", "background-color:aquamarine;", styles.toString());

    }

    public void testSetStyles() {
        Styles styles = new Styles();
        styles.setStyle("background", "blue");
        styles.setStyle("foreground", "red");
        log.info(styles);
        assertEquals("Style s/b", "background:blue;foreground:red;", styles.toString());

        Button button1 = new Button("test1");
        button1.setContent("Some buttom text");
        button1.setStyles(styles);
        log.info(button1);
        assertEquals("Button1 s/b", "<button name=\"test1\" style=\"background:blue;foreground:red;\">Some buttom text</button>", button1.toString());

        Button button2 = new Button("test2");
        button2.setContent("Some buttom text");
        button2.setAttribute("style","other:blah;");
        button2.setStyles(styles.toString());
        log.info(button2);

        assertEquals("Button2 s/b", "<button name=\"test2\" style=\"background:blue;foreground:red;other:blah;\">Some buttom text</button>", button2.toString());
    }

    public void testResetStyles() {
        Styles styles = new Styles();
        styles.setStyle("background", "blue");
        styles.setStyle("foreground", "red");
        log.info(styles);
        assertEquals("Style s/b", "background:blue;foreground:red;", styles.toString());

        Button button1 = new Button("test1");
        button1.setContent("Some buttom text");
        button1.setStyle("moo","junk");
        button1.resetStyles(styles);
        log.info(button1);
        assertEquals("Button1 s/b", "<button name=\"test1\" style=\"background:blue;foreground:red;\">Some buttom text</button>", button1.toString());

        Button button2 = new Button("test2");
        button2.setContent("Some buttom text");
        button1.setStyle("moo","junk");
        button2.resetStyles(styles.toString());
        log.info(button2);

        assertEquals("Button2 s/b", "<button name=\"test2\" style=\"background:blue;foreground:red;\">Some buttom text</button>", button2.toString());
    }


}
